#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import traceback
import stqe.host.tc
import libsan.host.lvm as lvm
import libsan.host.loopdev as loopdev
import libsan.host.linux as linux
from stqe.host.lvm import check_lv_expected_value
from six.moves import range

TestObj = None

loop_dev = {}

vg_name = "testvg"


def _lvs_test():
    TestObj.tok("lvcreate -l1 -T %s/pool1" % vg_name)
    # check_lv_expected_value uses lvs -o help to view the full list of columns available.

    check_lv_expected_value(TestObj, "pool1", vg_name, {"thin_count": "0"})

    # Create a thin LV
    TestObj.tok("lvcreate -V100m -T %s/pool1 -n lv1" % vg_name)
    check_lv_expected_value(TestObj, "pool1", vg_name, {"thin_count": "1"})
    check_lv_expected_value(TestObj, "pool1", vg_name, {"lv_name": "pool1"})
    check_lv_expected_value(TestObj, "pool1", vg_name, {"lv_size": "4.00m"})
    check_lv_expected_value(TestObj, "pool1", vg_name, {"lv_metadata_size": "4.00m"})
    # Since RHEL6.7 lvm2 package, adding 'device (o)pen' bit for lv_attr - 'twi-aotz--', previous is 'twi-a-tz--'.
    # From Zdenek feedback: there is difference we now display 'open' state for thin-pool in use.
    # While in past we have been showing 'open' state for fake thin-pool - now
    # we display 'correct' open stat for read thin-pool device.
    # So once you activate thin volume - thin-pool gets opened.
    # Thus this change should be seen as bugfix, where older versions were
    # basically showing thin pool as unopened device.
    check_lv_expected_value(TestObj, "pool1", vg_name, {"lv_attr": "twi-aotz--"})
    check_lv_expected_value(TestObj, "pool1", vg_name, {"modules": "thin-pool"})
    check_lv_expected_value(TestObj, "pool1", vg_name, {"metadata_lv": "[pool1_tmeta]"})
    check_lv_expected_value(TestObj, "pool1", vg_name, {"data_lv": "[pool1_tdata]"})
    TestObj.tok("lvs %s/pool1 -o+metadata_percent" % vg_name)
    check_lv_expected_value(TestObj, "pool1", vg_name, {"chunksize": "64.00k"})
    check_lv_expected_value(TestObj, "pool1", vg_name, {"transaction_id": "1"})

    check_lv_expected_value(TestObj, "lv1", vg_name, {"pool_lv": "pool1"})
    check_lv_expected_value(TestObj, "lv1", vg_name, {"lv_name": "lv1"})
    check_lv_expected_value(TestObj, "lv1", vg_name, {"lv_size": "100.00m"})
    check_lv_expected_value(TestObj, "lv1", vg_name, {"lv_attr": "Vwi-a-tz--"})
    # Kernel device-mapper modules required for this LV.
    # Since RHEL6.6, the order changed to 'thin,thin-pool', before was 'thin-pool,thin'
    check_lv_expected_value(TestObj, "lv1", vg_name, {"modules": "thin,thin-pool"})

    TestObj.tok("lvs -a %s | egrep '\[pool1_tdata\]\s+%s\s+Twi-ao----'" % (vg_name, vg_name))
    TestObj.tok("lvs -a %s | egrep '\[pool1_tmeta\]\s+%s\s+ewi-ao----'" % (vg_name, vg_name))
    TestObj.tok("lvs -a %s | egrep 'Meta%%'" % vg_name)
    TestObj.tok("lvs -a %s | egrep 'Data%%'" % vg_name)

    # Create another thin LV
    TestObj.tok("lvcreate -V100m -T %s/pool1 -n lv2" % vg_name)
    check_lv_expected_value(TestObj, "pool1", vg_name, {"transaction_id": "2"})
    check_lv_expected_value(TestObj, "pool1", vg_name, {"thin_count": "2"})
    # Since RHEL6.6, the value changed to 'zero', before was '1'
    check_lv_expected_value(TestObj, "pool1", vg_name, {"zero": "zero"})

    # Snapshot
    TestObj.tok("lvcreate -s %s/lv1 -n snap1" % vg_name)
    check_lv_expected_value(TestObj, "pool1", vg_name, {"transaction_id": "3"})
    check_lv_expected_value(TestObj, "pool1", vg_name, {"thin_count": "3"})
    check_lv_expected_value(TestObj, "snap1", vg_name, {"origin": "lv1"})
    check_lv_expected_value(TestObj, "snap1", vg_name, {"lv_name": "snap1"})
    check_lv_expected_value(TestObj, "snap1", vg_name, {"lv_size": "100.00m"})
    check_lv_expected_value(TestObj, "snap1", vg_name, {"lv_attr": "Vwi---tz-k"})
    # Since RHEL6.6, the order changed to 'thin,thin-pool', before was 'thin-pool,thin'
    check_lv_expected_value(TestObj, "snap1", vg_name, {"modules": "thin,thin-pool"})

    # warning messages for thin LV
    # thin pool autoextend
    # Check if warning message is given once the threshold of 80 and 85 are exceeded
    TestObj.tok("grep -E \"^\W+thin_pool_autoextend\" /etc/lvm/lvm.conf")
    TestObj.tok("lvcreate -l25 -V84m -T %s/pool2 -n lv3" % vg_name)
    TestObj.tok("dd if=/dev/zero of=/dev/%s/lv3 bs=1M count=84" % vg_name)
    check_lv_expected_value(TestObj, "lv3", vg_name, {"data_percent": "100.00"})
    check_lv_expected_value(TestObj, "pool2", vg_name, {"data_percent": "84.00"})
    # Wait some time so the message is recorded on system
    linux.sleep(30)
    # on RHEL-6.7 the message is: Thin tsvg-pool-tpool is now 84% full.
    # RHEL-6.8 is: Thin pool tsvg-pool2-tpool data is now 84.00% full.
    if not TestObj.tok("journalctl -n 200 | grep '%s-pool2-tpool .*is now 84.*%% full'" % vg_name):
        # dump the logs to help investigate any problem
        TestObj.trun("journalctl -n 200")
    TestObj.tok("lvextend -L88m %s/lv3" % vg_name)
    TestObj.tok("dd if=/dev/zero of=/dev/%s/lv3 bs=1M count=88" % vg_name)
    check_lv_expected_value(TestObj, "lv3", vg_name, {"data_percent": "100.00"})
    check_lv_expected_value(TestObj, "pool2", vg_name, {"data_percent": "88.00"})
    linux.sleep(30)
    if not TestObj.tok("journalctl -n 200 | grep '%s-pool2-tpool .*is now 88.*%% full'" % vg_name):
        # dump the logs to help investigate any problem
        TestObj.trun("journalctl -n 200")
    TestObj.tok("lvs %s/pool2 -o+metadata_percent" % vg_name)

    # Check warning level without resizing the LV
    TestObj.tok("lvcreate -L100m -V100m -T %s/pool3 -n lv4" % vg_name)
    thresholds = ["81", "86", "91", "96"]
    for threshold in thresholds:
        TestObj.tok("dd if=/dev/zero of=/dev/%s/lv4 bs=1M count=%s" % (vg_name, threshold))
        check_lv_expected_value(TestObj, "pool3", vg_name, {"data_percent": "%s.00" % threshold})
        linux.sleep(30)
        if not TestObj.tok("journalctl -n 200 | grep '%s-pool3-tpool .*is now %s.*%% full'" % (vg_name, threshold)):
            # dump the logs to help investigate any problem
            TestObj.trun("journalctl -n 200")

    # test lvs with invalid option
    TestObj.tnok("lvs -o +invalid_option %s/lv1 2>/dev/null" % vg_name)

    TestObj.trun("lvs -a %s" % vg_name)


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting lvs Thin Provisioning test")
    print(80 * "#")

    # Create 4 devices
    for dev_num in range(1, 5):
        new_dev = loopdev.create_loopdev(size=128)
        if not new_dev:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False
        loop_dev["dev%s" % dev_num] = new_dev

    pvs = ""
    for dev in loop_dev.keys():
        pvs += " %s" % (loop_dev[dev])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    _lvs_test()

    return True


def _clean_up():
    # TestObj.trun("lvremove -ff %s" % vg_name)
    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    try:
        start_test()
    except Exception as e:
        traceback.print_exc()
        TestObj.tfail("There was some problem while running the test (%s)" % e)
        print(e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.vdo import VDO
from stqe.host.atomic_run import atomic_run, parse_ret, Logger
from stqe.host.vdo import get_vdo_args, minimum_slab_size, get_underlying_device
from stqe.host.fmf_tools import get_env_args, read_env, get_func_from_string
from stqe.host.persistent_vars import write_var, read_var
from libsan.host.linux import run
import subprocess
import time
import signal
from functools import partial


def write_data(args, errors, extra):
    name = read_env("fmf_name")

    if "create_vdo" in name or ("create/success" in name):
        old_device = read_var('VDO_DEVICE')
        if old_device:
            previous_devices = read_var('VDO_OLD_DEVICES')
            if previous_devices:
                previous_devices.append(old_device)
            else:
                previous_devices = [old_device]
            atomic_run("Writing var VDO_OLD_DEVICES with previous devices.",
                       command=write_var,
                       var={'VDO_OLD_DEVICES': previous_devices},
                       errors=errors)
        else:
            print("INFO: Did not find any previous device.")

        atomic_run("Replacing var VDO_DEVICE with newly created device.",
                   command=write_var,
                   var={'VDO_DEVICE': "/dev/mapper/%s" % args["name"]},
                   errors=errors)

    if "remove_vdo" in name and extra["device"]:
        if len(extra['previous_devices']) > 0:
            atomic_run("Writing var VDO_OLD_DEVICES with previous devices.",
                       command=write_var,
                       var={'VDO_OLD_DEVICES': extra['previous_devices']},
                       errors=errors)
        atomic_run("Replacing var VDO_DEVICE with underlying device.",
                   command=write_var,
                   var={'VDO_DEVICE': extra["device"]},
                   errors=errors)

    if "create_vg" in name:
        atomic_run("Writing var VG_NAME",
                   command=write_var,
                   var={'VG_NAME': args['vg_name']},
                   errors=errors)

    if "mkdir" in name:
        atomic_run("Writing var FS_DIR",
                   command=write_var,
                   var={'FS_DIR': args['cmd'].split()[-1]},
                   errors=errors)

    if "create_lv" in name:
        atomic_run("Writing var %s" % args['lv_name'].upper(),
                   command=write_var,
                   var={args['lv_name'].upper(): args['lv_name']},
                   errors=errors)

    if all([True if x in name else False for x in ["create", "data", "file"]]):
        atomic_run("Writing var DATA_FILE",
                   command=write_var,
                   var={"DATA_FILE": args['cmd'].split("of=")[1].split()[0]},
                   errors=errors)

    # The DM table might be left over, better ensure it is removed now
    if "setup/remove_vdo" in read_env("fmf_name"):
        sleep_time = 60
        atomic_run("Sleeping for %s seconds" % sleep_time, command=time.sleep, secs=sleep_time, errors=errors)
        atomic_run("Checking device mapper table", command=run, cmd="dmsetup table", errors=errors)
        atomic_run("Checking /dev/mapper dir.", command=run, cmd="ls -la /dev/mapper", errors=errors)
        atomic_run("Removing DM device.", command=run, cmd="dmsetup remove %s" % args['name'], errors=[])

    return


def interrupt_command(message, command_string, kill_delay):
    logger = Logger()
    logger.info("%s, command: '%s'." % (message, command_string))
    p = subprocess.Popen(command_string, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    time.sleep(kill_delay)
    p.send_signal(signal.SIGINT)

    # Print the whole stdout and stderr
    out = ""
    err = ""
    for o in iter(partial(p.stdout.readline), b""):
        out += o.decode('ascii', 'ignore') if isinstance(o, bytes) else o
    for e in iter(partial(p.stderr.readline), b""):
        err += e.decode('ascii', 'ignore') if isinstance(e, bytes) else e
    print(out)
    logger.info("Killed the process after delay %ss" % kill_delay)
    print(err)

    time.sleep(5)
    return


def call_func(args, command, errors):
    extra = dict()
    if command == "create":
        args['message'] += " %s." % args['device']

    if "slab_size" in args and args["slab_size"] == "minimum":
        args['slab_size'] = minimum_slab_size(args['device'], default_to_2g=False)

    if command == "remove":
        previous_devices = read_var('VDO_OLD_DEVICES')
        if previous_devices:
            extra['device'] = previous_devices.pop()
            extra['previous_devices'] = previous_devices
        else:
            extra['device'] = get_underlying_device(args['name'])
            extra['previous_devices'] = []
        if not extra['device']:
            print("WARN: Could not find underlying device.")

    if command == "libsan.host.lvm.vg_create" or command == "libsan.host.lvm.lv_create":
        # need to rescan cache when working with lvm, VDO is not good at it.
        atomic_run("Rescanning cache",
                   command=run,
                   cmd="pvscan --cache",
                   errors=errors)

    if command == "libsan.host.lvm.lv_create":
        # Need to recalculate size if given -V X%FREE
        for i, option in enumerate(args['options']):
            if "%" not in option:
                continue
            pct = option.split("%")[0].split()[1]
            ret, data = atomic_run(message="Getting thinpool size.",
                                   command=run,
                                   cmd="lvs %s --noheadings --o size --units g" % args['vg_name'],
                                   return_output=True,
                                   errors=errors)
            try:
                args['options'][i] = "-V %sG " % ((float(pct) / 100) * float(data.strip()[:-1]))
            except ValueError as e:
                print(e)
                print("DEBUG: Got '%s' from command as data." % data)

    if command == "libsan.host.lvm.pv_remove":
        if args["pv_name"].startswith("/dev/mapper"):
            # need to get device in format /dev/vg/lv, format dm-X causes issues with pvremove libsan func
            ret, data = run("ls -la /dev/mapper | grep %s" % args["pv_name"].split("/").pop(), return_output=True)
            if data:
                device = data.split("->")[0].split().pop().strip().split("/").pop().replace("--", "-").split("-")
                if len(device) == 2:
                    args["pv_name"] = "/dev/%s/%s" % ("-".join(device[:-1]), device[-1])

    return extra


def setup():
    errors = []

    vdo = VDO(disable_check=True)
    args = get_vdo_args(vdo_object=vdo)

    # some extra args for some setups
    args.update(get_env_args(["prefix", "pv_name", "vg_name", "lv_name", "options", "cmd", "path",
                              "kill_delay", "command_string"],
                             read_var("FILE_NAMES")))

    command = args.pop('command')

    extra = call_func(args, command, errors)

    if command == "interrupt_command":
        def complete_command_string(cmd, args):
            cmd_string = cmd.pop(0)
            for param in cmd:
                param = param.split("=")
                cmd_string += " %s=%s" % (param[0], args[param[1]])
            return cmd_string

        command_string = complete_command_string(args["command_string"], args)
        interrupt_command(args["message"], command_string, args["kill_delay"])
    else:
        args['command'] = get_func_from_string(vdo, command)
        atomic_run(errors=errors,
                   **args)

    write_data(args, errors, extra)

    return errors


if __name__ == "__main__":
    errs = setup()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var
from libsan.host.qemu_img import delete_image


def delete_qcow():
    errors = []

    name = read_var("IMAGE_NAME")
    path = read_var("IMAGE_PATH")

    atomic_run("Deleting qcow image %s" % name,
               name=name,
               image_path=path,
               command=delete_image,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = delete_qcow()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.scsi import get_free_disks
from stqe.host.persistent_vars import write_var, read_env
from os import environ
from time import sleep


def get_sync_status(raid_name):
    ret, status = run(return_output=True,
                      cmd="mdadm --detail /dev/md/%s" % raid_name,
                      verbose=False)
    if ret != 0:
        print("WARN: Could not get details on raid %s" % raid_name)
        return None, None
    lines = status.splitlines()
    for line in lines:
        if "State :" in line:
            states = line.split(" : ")[1].split(", ")
            if len(states) == 1 and states[0].strip() == "clean":
                return "clean", True
        elif "Resync Status :" in line or "Rebuild Status :" in line:
            status = line
            return status[0].strip(), False
    print("WARN: It should not get here!")
    return None, None


def wait_for_resync(raid_name, delay=60):
    while 1:
        status, done = get_sync_status(raid_name)
        if done:
            return True
        elif not done and status is not None:
            print("INFO: %s." % status)
        elif done is None and status is None:
            return False
        sleep(delay)


def setup_raid(level, raid_name, wait_resync=False):
    print("INFO: Setting up RAID level %s." % level)
    errors = []

    disks = atomic_run(message="Getting free disks for RAID",
                       command=get_free_disks,
                       errors=errors)
    if disks is None:
        exit(1)
    disks = disks.keys()
    ret, lsblk = atomic_run(message="Listing block devices to check they are really free.",
                            command=run,
                            cmd="lsblk",
                            return_output=True,
                            errors=errors)
    # remove disks that are in use (find only partitioned ones)
    values = [x.strip() for line in lsblk.splitlines() for x in line.split()]
    for disk in [disk for value in values for disk in disks if disk != value and disk in value]:
        disks.remove(disk)

    disk_paths = ["/dev/" + j for j in disks]
    print("Found these free disks: %s" % " ".join(disk_paths))
    for disk in disk_paths:
        atomic_run("Zeroing superblock of disk %s." % disk,
                   command=run,
                   cmd="dd if=/dev/zero of=%s bs=4k count=2" % disk,
                   errors=errors)

    atomic_run("Creating raid %s level %s" % (raid_name, level),
               cmd="mdadm --create %s --level=%s --run --raid-devices=%s %s" %
                   (raid_name, level, len(disk_paths), " ".join(disk_paths)),
               command=run,
               errors=errors)

    run(cmd="mdadm --detail /dev/md/%s" % raid_name,
        verbose=True)

    print(wait_resync)
    if wait_resync:
        # Wait for resync if it is necessary
        atomic_run("Waiting for resync of raid %s" % raid_name,
                   raid_name=raid_name,
                   command=wait_for_resync,
                   errors=errors)

    atomic_run("Writing var VDO_DEVICE",
               command=write_var,
               var={'VDO_DEVICE': "/dev/md/%s" % raid_name},
               errors=errors)

    atomic_run("Writing var RAID_DISKS",
               command=write_var,
               var={'RAID_DISKS': " ".join(disk_paths)},
               errors=errors)

    atomic_run(message="Listing block devices.",
               command=run,
               cmd="lsblk",
               errors=errors)

    # sometimes there is some mess on the dist, clean it up
    atomic_run(message="Cleaning superblock of %s" % raid_name,
               command=run,
               cmd="dd if=/dev/zero of=/dev/md/%s bs=4K count=1" % raid_name,
               errors=errors)

    return errors


if __name__ == "__main__":
    level = 0
    try:
        level = int(environ['fmf_raid_level'])
    except KeyError:
        pass
    except TypeError:
        print("FMF metadata raid_level must be int.")
        exit(1)
    raid_name = "vdo_raid"

    wait_resync = False
    try:
        wait_resync = read_env("fmf_wait_resync")
    except KeyError:
        pass

    errs = setup_raid(level, raid_name, wait_resync=wait_resync)
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from os import environ
from libsan.host.linux import dist_release


def signatures(modules):
    # Check signatures of given modules.
    errors = []
    arguments = []

    if float(dist_release()) > 7.0:
        print("WARN: On modules are not signed yet on RHEL 8!")
        # later use keyring %:.builtin_trusted_keys when they get signed
        return errors

    if not isinstance(modules, list):
        return ["Signatures test requires modules parameter as list type, got %s" % type(modules)]

    for module in modules:
        cmd = "keyctl list %:.system_keyring | grep $(modinfo " + str(module) + " | grep sig_key " + \
              "| awk '{print tolower($2)}' | sed 's/\://g')"
        arguments += [
            # signature checking
            {'message': "Finding sig_key of module '%s'." % module, 'command': run,
             'cmd': 'modinfo %s | grep sig_key' % module},
            {'message': "Getting more info on module '%s' signature." % module, 'command': run,
             'cmd': cmd}
        ]

    for argument in arguments:
        atomic_run(errors=errors,
                   **argument)

    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = signatures(["uds", "kvdo"])
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from os import environ
from stqe.host.lsm import yield_lsm_config


def nsf_client_auth_success():
    errors = []
    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)
        atomic_run("Using LSM to list NFS_CLIENT_AUTH with protocol %s" % config['protocol'],
                   command=lsm.list,
                   lsm_type="NFS_CLIENT_AUTH",
                   errors=errors)
    return errors


def nsf_client_auth_fail():
    errors = []
    arguments = [
        {'sys': 'false_id'},
        {'pool': 'false_id'},
        {'vol': 'false_id'},
        {'disk': 'false_id'},
        {'ag': 'false_id'},
        {'fs': 'false_id'},
        {'nfs_export': 'false_id'},
        {'tgt': 'false_id'}
    ]
    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)
        for argument in arguments:
            atomic_run("Trying to fail using LSM to list NFS_CLIENT_AUTH with protocol %s" % config['protocol'],
                       errors=errors,
                       command=lsm.list,
                       lsm_type="NFS_CLIENT_AUTH",
                       success=False,
                       expected_ret=2,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = nsf_client_auth_success()
    if int(environ['fmf_tier']) == 2:
        errs = nsf_client_auth_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


# fs_delete_success is part of fs_create_success() in this same directory

def access_group_delete_fail():
    errors = []

    init_id = read_var("LSM_INIT_ID")
    sys_id = read_var("LSM_SYS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail deleting AG without paramethers with protocol %s" % config['protocol'],
             'command': lsm.access_group_delete},
            {'message': "Trying to fail deleting AG with WRONG AG with protocol %s" % config['protocol'],
             'command': lsm.access_group_delete, 'ag': "WRONG"},
            {'message': "Trying to fail deleting AG with init ID instead of AG ID name with protocol %s" % config['protocol'],
             'command': lsm.access_group_delete, 'ag': init_id},
            {'message': "Trying to fail deleting AG with sys ID instead of AG ID name with protocol %s" % config['protocol'],
             'command': lsm.access_group_delete, 'ag': sys_id},
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 2:
        errs = access_group_delete_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
import os
from stqe.host.atomic_run import parse_ret
from libsan.host.cmdline import run
from libsan.host.linux import dist_ver, dist_name


def tcmu_runner_install():
    errors = []

    repo = os.environ["fmf_tcmu_repo_link"]  # "https://github.com/open-iscsi/tcmu-runner.git"
    path = os.environ["fmf_tcmu_path"]
    if os.path.isdir(path):
        print("Directory %s already exists! Skipping installation." % path)
        return errors

    packages = "cmake make gcc libnl3 glib2 zlib kmod libnl3-devel glib2-devel zlib-devel " \
               "kmod-devel librados2 librados2-devel librbd1 librbd1-devel qemu-img"

    cmd = "git clone %s %s" % (repo, path)
    ret = run(cmd)
    if ret != 0:
        msg = "FAIL: Could not clone repo %s to directory %s" % (repo, path)
        print(msg)
        errors.append(msg)
        return errors

    cmd = "dnf install -y %s " % packages
    if dist_name() != 'Fedora':
        cmd += "--enablerepo=rhel-buildroot"
        if float(dist_ver() < 8.0):
            cmd = "yum install -y %s" % packages

    ret = run(cmd)
    if ret != 0:
        msg = "FAIL: Could not install all packages!"
        print(msg)
        errors.append(msg)
        return errors

    current_directory = os.path.abspath(os.curdir)
    os.chdir(path)

    cmd = "cmake . -Dwith-glfs=false -DSUPPORT_SYSTEMD=ON -DCMAKE_INSTALL_PREFIX=/usr"
    ret = run(cmd)
    if ret != 0:
        msg = "FAIL:Command 'cmake' exited with retcode=%s" % ret
        print(msg)
        errors.append(msg)
        return errors

    os.chdir(current_directory)

    cmd = "make --directory=%s" % path
    ret = run(cmd)
    if ret != 0:
        msg = "FAIL:Command 'make install' exited with retcode=%s" % ret
        print(msg)
        errors.append(msg)
        return errors

    cmd = "make install --directory=%s" % path
    ret = run(cmd)
    if ret != 0:
        msg = "FAIL:Command 'make install' exited with retcode=%s" % ret
        print(msg)
        errors.append(msg)
        return errors

    return errors


if __name__ == "__main__":
    errs = tcmu_runner_install()
    exit(parse_ret(errs))



# Copyright (C) 2018 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

"""lsm.py: Module with test specific method for libstoragemgmt."""

from __future__ import absolute_import, division, print_function, unicode_literals
import re
from stqe.host.persistent_vars import read_var, read_env
from stqe.host.fmf_tools import get_env_args


def check_ssl(protocol):
    """
    Checks if fmf_protocol is ssl/no_ssl and limits the protocol by this
    :param protocol:
    :return:
    """
    try:
        ssl = read_env("fmf_protocol")
    except KeyError:
        # Protocol not limited
        return True
    if ssl == "ssl" and "ssl" in protocol or ssl == "no_ssl" and "ssl" not in protocol:
        return True
    return False


def yield_lsm_config():
    config = {
        'protocols': list(),
        'username': None,
        'password': None,
        'target': None
    }

    for conf in config.keys():
        try:
            config[conf] = read_var("LSM_" + conf.upper())
        except IOError:
            pass

    try:
        if not isinstance(config['protocols'], list):
            raise AttributeError
    except AttributeError:
        print("ERROR: Protocols must be list, got %s, type '%s'." % (config['protocols'], type(config['protocols'])))
        exit(1)

    protocols = config.pop('protocols')
    for protocol in protocols:
        if not check_ssl(protocol):
            continue
        config['protocol'] = protocol
        yield config


def get_data_from_script_output(data, id="ID"):
    split_string = re.match("(-*)", data).group()
    try:
        items = [x for x in data.split(split_string) if id in x]
    except ValueError as e:
        print(repr(e))
        return None
    data = dict()
    for item in items:
        for line in item.splitlines():
            if len(line) < 2:
                continue
            if line.startswith(id):
                # luckily ID is always first
                item_id = line.split("|").pop().strip()
                data[item_id] = dict()
            else:
                line_data = line.split("|")
                data[item_id][line_data[0].strip()] = line_data[1].strip()
    return data


def get_local_disk_data(data):
    return get_data_from_script_output(data, id="Path")


def get_id_from_name(data, name, field='Name', item_id="ID"):
    data = get_data_from_script_output(data, id=item_id)
    for line in data:
        if data[line][field] == name:
            return line
    print("FAIL: Could not find item named '%s'." % name)
    return None


def get_data_from_id(data, name, field='Name', item_id="ID"):
    data = get_data_from_script_output(data, id=item_id)
    for line in data:
        if line != name:
            continue
        return data[line][field]
    print("FAIL: Could not find item named '%s'." % name)
    return None


def get_ag_id_from_name(data, name):
    return get_id_from_name(data, name)


def get_fs_id_from_name(data, name):
    return get_id_from_name(data, name)


def get_vol_id_from_name(data, name):
    return get_id_from_name(data, name)


def get_export_id_from_export_path(data, export_path):
    # policy is 'disabled' but cli takes 'disable'
    return get_id_from_name(data, export_path, field="Export Path")


def get_system_read_pct_of_sys(data, sys):
    return get_data_from_script_output(data)[sys]['Read Cache Percentage']


def get_cache_policy_from_id(data, vol_id, field):
    return translate_cache_policy(get_data_from_id(data, vol_id, field=field, item_id="Volume ID"))


def translate_cache_policy(policy):
    dictionary = {"Write Back": "WB",
                  "Write Through": "WT",
                  "Auto": "AUTO",
                  "Enabled": "enable",
                  "Disabled": "disable"}
    try:
        return dictionary[policy]
    except KeyError:
        return policy


def get_replace_dict():
    """
    Returns dict of keys to replace from fmf to libsan.host.lsm.LibStorageMgmt
    :return: dict
    """
    return {"ag_name": 'name',
            "vol_name": 'name',
            "fs_name": 'name',
            "snap_name": 'name',
            "rep_name": 'name'}


def get_lsm_args(lsm_object):
    """
    Returns dict of arguments passed from FMF env vars
    :return: dict
    """
    arguments = dict(lsm_object.arguments)
    # "name" has to be removed, as name is FMF name of the test at this point
    del (arguments["name"])
    return get_env_args(list(arguments) + list(get_replace_dict()),
                        read_var("FILE_NAMES"),
                        get_replace_dict())



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config, get_local_disk_data
from os import environ


def local_disk_fault_led_success():
    errors = []

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])
    ret, disks = atomic_run("Listing local disks",
                            command=lsm.local_disk_list,
                            return_output=True,
                            script=True,
                            errors=errors)
    data = get_local_disk_data(disks)
    if not data:
        print("WARN: Could not find any local disks, skipping.")
        return errors

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        for disk in data:
            if data[disk]["LED Status"] == "Unknown":
                print("WARN: Disk %s does not support LED changes." % disk)
                continue
            status = data[disk]["LED Status"]
            arguments = [
                {'message': "Switching fault LED ON with protocol %s" % config['protocol'],
                 'command': lsm.local_disk_ident_led_on},
                {'message': "Switching fault LED OFF with protocol %s" % config['protocol'],
                 'command': lsm.local_disk_ident_led_off}
            ]
            if status == "ON":
                arguments = reversed(arguments)
            for argument in arguments:
                atomic_run(path=disk,
                           errors=errors,
                           **argument)
    return errors


def local_disk_fault_led_on_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail to set fault led ON without path with protocol %s" % config['protocol'],
             'command': lsm.local_disk_fault_led_on},
            {'message': "Trying to fail to set fault led ON with wrong path with protocol %s" % config['protocol'],
             'command': lsm.local_disk_fault_led_on, 'path': "WRONG"}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


def local_disk_fault_led_off_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail to set fault led OFF without path with protocol %s" % config['protocol'],
             'command': lsm.local_disk_fault_led_off},
            {'message': "Trying to fail to set fault led OFF with wrong path with protocol %s" % config['protocol'],
             'command': lsm.local_disk_fault_led_off, 'path': "WRONG"}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = local_disk_fault_led_success()
    if int(environ['fmf_tier']) == 2:
        errs = local_disk_fault_led_on_fail()
        errs += local_disk_fault_led_off_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env

# SETUP
run("dmsetup create tst_zero --table '0 131072 zero'")
run("dmsetup create tst_error --table '0 131072 error';")
run("""dmsetup create tst_err_writes << EOF
 0 22480 linear /dev/mapper/tst_zero 0
 22480 32 delay /dev/mapper/tst_zero 22480 0 /dev/mapper/tst_error 22480 0
 22512 108560 linear /dev/mapper/tst_zero 22512
EOF""")
# TEST

error = list()
try:
    expected_out = read_env("fmf_expected_out")
except KeyError:
    expected_out = "+0 records in"

try:
    expected_ret = read_env("fmf_expected_ret")
except KeyError:
    expected_ret = 0

atomic_run(message="Testing aligned",
           command=run,
           cmd="dd if=/dev/mapper/tst_err_writes of=/dev/mapper/tst_err_writes bs=4M iflag=direct oflag=direct;",
           expected_ret=expected_ret, expected_out=expected_out, errors=error)

atomic_run(message="Testing misaligned.",
           command=run,
           cmd="dd if=/dev/mapper/tst_err_writes of=/dev/mapper/tst_err_writes bs=2M iflag=direct oflag=direct",
           expected_ret=expected_ret, expected_out=expected_out, errors=error)
# CLEANUP
run("dmsetup remove tst_err_writes;")
run("dmsetup remove tst_error")
run("dmsetup remove tst_zero")

exit(parse_ret(error))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

#
# Bugs related:
#   BZ1319937 - pool created without zeroing the first 4KiB (--zero n) can not have meta corrupted and then repaired 
#

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.cmdline import run
from libsan.host.lvm import vg_create, vg_remove
from libsan.host.linux import install_package
import stqe.host.tc
import sys
from six.moves import range

TestObj = None

# global vg_name
# global pool_name

loop_dev0 = None
loop_dev1 = None

vg_name = "VG"
pool_name = "POOL"
lv_meta_swap = "meta_swap"


def main():
    global TestObj

    install_package("lvm2")

    TestObj = stqe.host.tc.TestClass()
    _clean_up()

    test_without_zero()
    _clean_up()

    test_with_zero()
    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


def start(zero):
    global TestObj

    global loop_dev0, loop_dev1

    # create loop device of 10G
    loop_dev0 = create_loopdev("loop0", 5 * 1024)
    if not loop_dev0:
        TestObj.tfail("Could not create loop0 device")
    loop_dev1 = create_loopdev("loop1", 5 * 1024)
    if not loop_dev1:
        TestObj.tfail("Could not create loop1 device")

    pv_devs = loop_dev0 + " " + loop_dev1
    if not vg_create(vg_name, pv_devs):
        TestObj.tfail("Could not create VG \"%s\"" % vg_name)

    print("INFO: Creating LVs")
    TestObj.tok("lvcreate --thinpool %s --zero %s -L 1G --poolmetadatasize 4M %s"
                % (pool_name, zero, vg_name))
    TestObj.tok("lvcreate --virtualsize 1G -T %s/%s -n origin" % (vg_name, pool_name))
    TestObj.tok("lvcreate --virtualsize 1G -T %s/%s -n other1" % (vg_name, pool_name))
    TestObj.tok("lvcreate --virtualsize 1G -T %s/%s -n other2" % (vg_name, pool_name))
    TestObj.tok("lvcreate --virtualsize 1G -T %s/%s -n other3" % (vg_name, pool_name))
    TestObj.tok("lvcreate --virtualsize 1G -T %s/%s -n other4" % (vg_name, pool_name))
    TestObj.tok("lvcreate -k n -s /dev/%s/origin -n snap" % vg_name)
    TestObj.tok("lvs -a -o +devices")
    print("INFO: tmeta is using:")
    TestObj.tok("lvs -a -o name,devices | grep %s_tmeta | awk {'print $2'}" % pool_name)

    print("INFO: Corrupting metadata")
    run("dd if=/dev/urandom of=/dev/mapper/%s-%s_tmeta count=512 seek=4096 bs=1" % (vg_name, pool_name))
    TestObj.tok("vgchange -an %s" % vg_name)

    print("INFO: Sanity checking pool device (%s) metadata" % pool_name)
    TestObj.tok("lvchange -an --yes --select 'lv_name=%s || pool_lv=%s'" % (pool_name, pool_name))

    print("INFO: create tmp lv in order to swap the existing metadata device")
    TestObj.tok("lvcreate -n %s -L 4M %s" % (lv_meta_swap, vg_name))

    print("INFO: swap the tmp device with the existing metadata device")
    TestObj.tok("lvconvert --yes --thinpool %s/%s --poolmetadata %s/%s" % (vg_name, pool_name, vg_name, lv_meta_swap))

    print("INFO: do the actual check")
    TestObj.tok("lvchange -ay %s/%s" % (vg_name, lv_meta_swap))
    TestObj.tnok("thin_check /dev/mapper/%s-%s" % (vg_name, lv_meta_swap))

    print("INFO: swap the devices back to their original positions")
    TestObj.tok("lvchange -an %s/%s" % (vg_name, lv_meta_swap))
    TestObj.tok("lvconvert --yes --thinpool %s/%s --poolmetadata %s/%s" % (vg_name, pool_name, vg_name, lv_meta_swap))

    print("INFO: remove tmp device")
    TestObj.tok("lvremove %s/%s" % (vg_name, lv_meta_swap))
    TestObj.tnok("lvchange -ay --yes --select 'lv_name=%s || pool_lv=%s'" % (pool_name, pool_name))

    print("INFO: Swap in new _tmeta device using lvconvert --repair")
    TestObj.tok("lvconvert --yes --repair %s/%s %s" % (vg_name, pool_name, loop_dev1))
    print("INFO: tmeta is using:")
    TestObj.tok("lvs -a -o name,devices | grep %s_tmeta | awk {'print $2'}" % pool_name)
    TestObj.tok("vgchange -ay %s" % vg_name)

    print("INFO: Sanity checking pool device (%s) metadata again" % pool_name)
    TestObj.tok("lvchange -an --yes --select 'lv_name=%s || pool_lv=%s'" % (pool_name, pool_name))

    print("INFO: create tmp lv in order to swap the existing metadata device")
    TestObj.tok("lvcreate -n %s -L 4M %s" % (lv_meta_swap, vg_name))

    print("INFO: swap the tmp device with the existing metadata device")
    TestObj.tok("lvconvert --yes --thinpool %s/%s --poolmetadata %s/%s" % (vg_name, pool_name, vg_name, lv_meta_swap))

    print("INFO: do the actual check")
    TestObj.tok("lvchange -ay %s/%s" % (vg_name, lv_meta_swap))
    TestObj.tok("thin_check /dev/mapper/%s-%s" % (vg_name, lv_meta_swap))

    TestObj.tok("lvchange -an %s/%s" % (vg_name, lv_meta_swap))
    TestObj.tok("lvconvert --yes --thinpool %s/%s --poolmetadata %s/%s" % (vg_name, pool_name, vg_name, lv_meta_swap))

    print("INFO: remove tmp device")
    TestObj.tok("lvremove %s/%s" % (vg_name, lv_meta_swap))
    TestObj.tok("lvchange -ay --yes --select 'lv_name=%s || pool_lv=%s'" % (pool_name, pool_name))

    _clean_up()

    # sys.exit(0)
    return True


def _clean_up():
    global vg_name, loop_dev0, loop_dev1

    vg_remove(vg_name, force=True)

    if loop_dev0:
        run("pvremove --yes --force --force %s" % loop_dev0)
        if delete_loopdev(loop_dev0):
            loop_dev0 = None

    if loop_dev1:
        run("pvremove --yes --force --force %s" % loop_dev1)
        if delete_loopdev(loop_dev1):
            loop_dev1 = None
    return True


def test_without_zero():
    for i in range(1, 6):
        print("############################################################")
        print("INFO: Running test WITHOUT zeroing first 4KiB. Interaction %d" % i)
        print("############################################################")
        start("n")


def test_with_zero():
    for i in range(1, 6):
        print("############################################################")
        print("INFO: Running test ZEROING first 4KiB. Interaction %d" % i)
        print("############################################################")
        start("y")


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var
from libsan.host.lio import TargetCLI
from os import environ


def create_backstore():
    errors = []

    arguments = {
        'name': None,
        'dev': None,
        'file_or_dev': None,
        'size': None,
        'writeback': None,
        'sparse': None,
        'wwn': None,
        'readonly': None,
        'nullio': None,
        'add_mapped_luns': None,
        'lun': None,
        'storage_object': None,
        'ip_address': None,
        'ip_port': None,
        'tag': None,
        'cfgstring': None,
        'hw_max_sectors': None,
        'control': None
    }

    backstore_type = environ["fmf_backstore_type"]

    for arg in arguments.keys():
        try:
            if arg == 'name':
                arguments[arg] = environ['fmf_backstore_name']
                continue
            if arg == "file_or_dev":
                arguments[arg] = "/var/tmp/" + environ["fmf_file_or_dev"]
                continue
            arguments[arg] = environ['fmf_' + arg]
        except KeyError:
            pass

    try:
        loopdev = environ['fmf_loopdev']
    except KeyError:
        loopdev = None
        pass

    if loopdev:
        arguments["dev"] = read_var("TARGETCLI_LOOPDEV_NAME")

    if backstore_type == "user:qcow":
        arguments["cfgstring"] = read_var("IMAGE_PATH") + "/" + read_var("IMAGE_NAME") + ".img"

    target = TargetCLI(path="/backstores/" + backstore_type)

    atomic_run("Creating backstore object with path %s" % target.path,
               command=target.create,
               errors=errors,
               **target.remove_nones(arguments)
               )

    atomic_run("Writing var TARGETCLI_%s_NAME" % backstore_type,
               command=write_var,
               var={backstore_type.upper() + "_NAME": arguments["name"]},
               errors=errors)

    if arguments["file_or_dev"]:
        atomic_run("Writing var FILEIO_FILE",
                   command=write_var,
                   var={"FILEIO_FILE": arguments["file_or_dev"]},
                   errors=errors)

    return errors


if __name__ == "__main__":
    errs = create_backstore()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from os import environ
from stqe.host.persistent_vars import read_var


def volume_access_group_success():
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        ret, data = atomic_run("Checking access group for volume %s with protocol %s." % (vol_id, config['protocol']),
                               command=lsm.volume_access_group,
                               vol=vol_id,
                               return_output=True,
                               errors=errors)
        if data is None or ag_id not in data:
            msg = "FAIL: Could not find access group %s link to volume %s" % (ag_id, vol_id)
            print(msg)
            errors.append(msg)
        return errors


def volume_access_group_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail checking access group for volume without any paramethers with protocol %s"
                        % config['protocol'], 'command': lsm.volume_access_group}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = volume_access_group_success()
    if int(environ['fmf_tier']) == 2:
        errs = volume_access_group_fail()
    exit(parse_ret(errs))



from __future__ import absolute_import, division, print_function, unicode_literals

# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

"""persistent_vars.py: Module to share variables between scripts using writing to file."""

import os
from libsan.host.cmdline import run


def get_persistent_files_dir():
    """
    :return: Directory of persistent vars.
    :rtype: string
    """
    return "/tmp/"


def get_persistent_vars_file_name():
    """
    :return: File name of persistent vars file.
    :rtype: string
    """
    return "FILE_NAMES"


def exists_persistent_vars_file():
    """
    :return: True if persistent vars filer exists, default /tmp/FILE_NAMES
    :rtype: bool
    """
    return os.path.exists(get_persistent_files_dir() + get_persistent_vars_file_name())


def _check_0_start(value):
    """
    Checks if value starts with 0 but is not 0. This prevents from changing '02314' but allows changing '0'
    :param value: value to check
    :type value: string
    :return: True if starts with 0 and is not 0, False otherwise
    :rtype: bool
    """
    return all([value.startswith("0"), not value == "0"])


def _read_to_float(value):
    """
    :param value: Value to try to change to float
    :type value: any, preferable string
    :return: (False, original value) or (True, float(value))
    :rtype: tuple
    """
    try:
        return (True, float(value)) if not all([_check_0_start(value), not value.startswith("0.")]) else (False, value)
    except ValueError:
        return False, value


def _read_to_int(value):
    """
    :param value: Value to try to change to integer
    :type value: any, preferable string
    :return: (False, original value) or (True, int(value))
    :rtype: tuple
    """
    try:
        return (True, int(value)) if not _check_0_start(value) else (False, value)
    except ValueError:
        return False, value


def _read_to_list(value):
    """
    This does not support nested lists!
    :param value: Value to try to change to list
    :type value: any, preferable string
    :return: (False, original value) or (True, list(value))
    :rtype: tuple
    """
    try:
        if value.startswith("["):
            return True, value[1:-1].split(", ")
        return False, value
    except AttributeError:
        return False, value


def _read_to_none(value):
    """
    :param value: Value to try to change to None
    :type value: any, preferable string
    :return: (False, original value) or (True, None)
    :rtype: tuple
    """
    if value == str(None):
        return True, None
    return False, value


def _read_to_bool(value):
    """
    Checks if value is "true" or "false" string, returns bool equivalent.
    :param value: Value to try to change to bool
    :type value: any, preferable string
    :return: (False, original value) or (True, True/False)
    :rtype: tuple
    """
    if value.lower() == "true":
        return True, True
    elif value.lower() == "false":
        return True, False
    return False, value


def _remove_quotes(value):
    """
    Removes any characters ' or " from any string
    :param value: Value to remove quotes from
    :type value: string
    :return: original string without quotes
    :rtype: string
    """
    return "".join([char for char in value if char not in ["\'", "\""]])


def recursive_read_value(value):
    """
    Given string tries to change type of the string to different types. Returns value of new type of possible.
    Supported new types: list, int, float, None, bool
    Example: str("21654") changes to int(21654)
    :param value: value to be retyped
    :type value: string
    :return: value of new type
    :rtype: any(string, list, int, float, None, bool)
    """
    value = _remove_quotes(value)
    for func in [_read_to_list, _read_to_int, _read_to_float, _read_to_none, _read_to_bool]:
        ret, value = func(value)
        if ret:
            if func == _read_to_list:
                return [recursive_read_value(val) for val in value]
            return value
    return value


def read_var(var):
    """
    :param var: variable name saved in file of the same name in get_persistent_files_dir() location
    :type var: string
    :return: Value saved in the file with proper type
    :rtype: any(_recursive_read_value)
    """
    if not os.path.isfile(get_persistent_files_dir() + var):
        if var != get_persistent_vars_file_name():
            print("WARN: File %s does not exist." % (get_persistent_files_dir() + var))
        return None
    with open(get_persistent_files_dir() + var, 'r') as f:
        value = f.read()
    return recursive_read_value(value)


def read_env(var):
    """
    This does not handle KeyError, it is intentional.
    :param var: os.environ variable
    :type var: string
    :return: value of environ variable
    :rtype: any(_recursive_read_value)
    """
    return recursive_read_value(os.environ[var])


def _write_to_string(value):
    """
    :param value: Value to try to change to string
    :type value: any
    :return: (False, original value) or (True, str(value))
    :rtype: tuple
    """
    try:
        return True, str(value)
    except ValueError:
        return False, value


def _write_from_list(value):
    """
    :param value: Value to try to change to list
    :type value: list
    :return: (False, original value) or (True, str(list(value)) without '' or "")
    :rtype: tuple
    """
    if isinstance(value, list):
        return True, "[%s]" % ", ".join([str(x) for x in value[:]])
    return False, value


def write_var(var):
    """
    :param var: variable to write to file in format {var_name: var_value}
    :type var: dict
    :return: 0 pass, 1 fail
    :rtype: int
    """
    if not isinstance(var, dict):
        print("FAIL: var manipulation requires var as a dict. {name: value}")
        return 1
    file_name = list(var.keys()).pop()
    write_file(file_name, list(var.values()).pop())
    add_file_to_list(file_name)
    return 0


def write_file(file_name, value):
    """
    :param file_name: File name to write to location get_persistent_files_dir() + file_name
    :type file_name: string
    :param value: value to write to the file
    :type value: dict
    :return: None
    :rtype: None
    """
    with open(get_persistent_files_dir() + file_name, 'w') as f:
        for func in (_write_from_list, _write_to_string):
            ret, value = func(value)
            if ret:
                break
        f.write(value)


def add_file_to_list(file_name):
    """
    This is for adding file names to persistent list so we can clean them later.
    :param file_name: name of the persistent vars file
    :type file_name: string
    :return: None
    :rtype: None
    """
    file_names = read_var(get_persistent_vars_file_name()) or []
    if file_name not in file_names:
        file_names.append(file_name)
    write_file(get_persistent_vars_file_name(), file_names)


def clean_var(var):
    """
    Cleans persistent var file from filesystem
    :param var: variable name to clean
    :type var: string
    :return: 0
    :rtype: int
    """
    os.remove(get_persistent_files_dir() + var)
    return 0


def clean_all_vars(prefix=""):
    """
    This uses list of persistent vars files from get_persistent_files_dir() and get_persistent_vars_file_name() and
    cleans them from filesystem.
    :param prefix: prefix to clean only some, for example "LSM_"
    :type prefix: string
    :return: 0
    :rtype: int
    """
    prefix = prefix or ""
    variables = get_persistent_var_names(prefix)
    if exists_persistent_vars_file() and get_persistent_vars_file_name() not in variables:
        variables.append(get_persistent_vars_file_name())
    print("INFO: Will clean these variables: \n\t%s" % "\n\t".join(variables))
    for var in variables:
        clean_var(var)
    return 0


def get_persistent_var_names(prefix=""):
    """
    Gets persistent vars from get_persistent_vars_file_name() file or filesystem in get_persistent_files_dir() location
    :param prefix: prefix to clean only some, for example "LSM_"
    :type prefix: string
    :return: List of persistent vars file names having the specified prefix
    :rtype: list
    """
    if exists_persistent_vars_file():
        variables = read_var(get_persistent_vars_file_name())
    else:
        _, variables = run("ls -la %s" % get_persistent_files_dir(), return_output=True, verbose=False)
        variables = [line.split().pop() for line in variables.splitlines()[3:]]
    return [value for value in variables if value.startswith(prefix)]



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import traceback
import stqe.host.tc
import libsan.host.lvm as lvm
import libsan.host.linux as linux
import libsan.host.loopdev as loopdev

from stqe.host.lvm import check_lv_expected_value
from six.moves import range

TestObj = None

loop_dev = {}

vg_name = "testvg"

lv_mnt = "/mnt/lv"
snap_mnt = "/mnt/snap"


def _extend_pool():
    global TestObj, vg_name

    TestObj.tok("lvcreate -l2 -T %s/pool1" % vg_name)
    TestObj.tok("lvcreate -i2 -l2 -T %s/pool2" % vg_name)

    pvs = []
    for value in loop_dev.values():
        pvs.append(value)

    for pool_num in range(1, 3):
        TestObj.tok("lvresize -l+2 -n %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name,
                                {"lv_size": "16.00m"})
        # Default unit is m
        TestObj.tok("lvresize -L+8 -n %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_size": "24.00m"})
        TestObj.tok("lvresize -L+8M -n %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_size": "32.00m"})

        # Need to use different devices as we are forcing allocation
        if pool_num == 1:
            # extend using some arbitary device
            TestObj.tok("lvresize -l+2 -n %s/pool%d %s" % (vg_name, pool_num, pvs[3]))
            check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_size": "40.00m"})
            # extend using specific range of physical extent
            TestObj.tok("lvresize -l+2 -n %s/pool%d %s:40:41" % (vg_name, pool_num, pvs[2]))
            check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_size": "48.00m"})
            TestObj.tok("pvs -ovg_name,lv_name,devices %s | grep '%s(40)'" % (pvs[2], pvs[2]))

            TestObj.tok("lvresize -l+2 -n %s/pool%d %s:35:37" % (vg_name, pool_num, pvs[1]))
            check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_size": "56.00m"})
            TestObj.tok("pvs -ovg_name,lv_name,devices %s | grep '%s(35)'" % (pvs[1], pvs[1]))

        else:
            # extend using some arbitary device
            TestObj.tok("lvresize -l+2 -n %s/pool%d %s %s" % (vg_name, pool_num, pvs[1], pvs[2]))
            check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_size": "40.00m"})

            TestObj.tok("lvresize -l+2 -n %s/pool%d %s:30-41 %s:20-31" % (vg_name, pool_num, pvs[1], pvs[2]))
            check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_size": "48.00m"})
            TestObj.tok("pvs -ovg_name,lv_name,devices %s | grep '%s(30)'" % (pvs[1], pvs[1]))
            TestObj.tok("pvs -ovg_name,lv_name,devices %s | grep '%s(20)'" % (pvs[2], pvs[2]))

        # set specific size
        TestObj.tok("lvresize -l16 -n %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_size": "64.00m"})
        TestObj.tok("lvresize -L72m -n %s/pool%d" % (vg_name, pool_num))
        check_lv_expected_value(TestObj, "pool%d" % pool_num, vg_name, {"lv_size": "72.00m"})

        # Using test option so size will not change
        TestObj.tok("lvresize -l+100%%FREE --test %s/pool%d" % (vg_name, pool_num))
        TestObj.tok("lvresize -l+10%%PVS --test %s/pool%d" % (vg_name, pool_num))
        # Extending thin LV based on VG does not make sense, but leaving this for now
        TestObj.tok("lvresize -l+10%%VG -t %s/pool%d" % (vg_name, pool_num))
        TestObj.tnok("lvresize -l+100%%VG -t %s/pool%d" % (vg_name, pool_num))
    TestObj.tok("lvremove -ff %s" % vg_name)


def _extend_thin_lv():
    global TestObj, vg_name

    TestObj.tok("lvcreate -l10 -V8m -T %s/pool1 -n lv1" % vg_name)
    TestObj.tok("lvcreate -i2 -l10 -V8m -T %s/pool2 -n lv2" % vg_name)

    for lv_num in range(1, 3):
        TestObj.tok("lvextend -l4 %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name,
                                {"lv_size": "16.00m"})
        # Default unit is m
        TestObj.tok("lvextend -L24 -n %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_size": "24.00m"})

        # Using test option so size will not change
        TestObj.tok("lvextend -l+100%%FREE --test %s/lv%d" % (vg_name, lv_num))
        TestObj.tok("lvextend -l+100%%PVS --test %s/lv%d" % (vg_name, lv_num))
        # Extending thin LV based on VG does not make sense, but leaving this for now
        TestObj.tok("lvextend -l+50%%VG -t %s/lv%d" % (vg_name, lv_num))
        TestObj.tok("lvextend -l+120%%VG -t %s/lv%d" % (vg_name, lv_num))

        if not linux.mkdir(lv_mnt):
            TestObj.tfail("Could not create %s" % lv_mnt)
            return False
        fs = linux.get_default_fs()

        lv_device = "/dev/mapper/%s-lv%d" % (vg_name, lv_num)
        if not linux.mkfs(lv_device, fs, force=True):
            TestObj.tfail("Could not create fs on %s" % lv_device)
            return False

        if not linux.mount(lv_device, lv_mnt):
            TestObj.tfail("Could not mount %s" % lv_device)
            return False
        TestObj.tok("dd if=/dev/urandom of=%s/lv%d bs=1M count=5" % (lv_mnt, lv_num))

        # extend FS
        TestObj.tok("lvextend -l+2 -r %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_size": "32.00m"})

        # snapshot
        snap_device = "/dev/mapper/%s-snap%d" % (vg_name, lv_num)
        TestObj.tok("lvcreate -K -s %s/lv%d -n snap%d" % (vg_name, lv_num, lv_num))
        if not linux.mkdir(snap_mnt):
            TestObj.tfail("Could not create %s" % snap_mnt)
            return False

        if not linux.mkfs(snap_device, fs, force=True):
            TestObj.tfail("Could not create fs on %s" % snap_device)
            return False

        if not linux.mount(snap_device, snap_mnt):
            TestObj.tfail("Could not mount %s" % snap_device)
            return False

        TestObj.tok("dd if=/dev/urandom of=%s/lv%d bs=1M count=5" % (snap_mnt, lv_num))
        # extend FS
        TestObj.tok("lvextend -l+2 -rf %s/snap%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "snap%d" % lv_num, vg_name, {"lv_size": "40.00m"})
        TestObj.trun("df -h %s" % snap_mnt)

        TestObj.tok("lvextend -L48 -rf %s/snap%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "snap%d" % lv_num, vg_name, {"lv_size": "48.00m"})
        TestObj.trun("df -h %s" % snap_mnt)

        if not linux.umount(lv_mnt):
            TestObj.tfail("Could not umount %s" % lv_device)
            return False
        if not linux.umount(snap_mnt):
            TestObj.tfail("Could not umount %s" % snap_device)
            return False
    TestObj.tok("lvremove -ff %s" % vg_name)


###########
# reduce
###########

def _reduce_pool():
    global TestObj, vg_name

    TestObj.tok("lvcreate -L400M -T %s/pool1" % vg_name)
    # reduce thin pool is not supported
    TestObj.tnok("lvresize -l-2 -n %s/pool1" % vg_name)
    check_lv_expected_value(TestObj, "pool1", vg_name, {"lv_size": "400.00m"})

    TestObj.tok("lvremove -ff %s" % vg_name)


def _reduce_thin_lv():
    global TestObj, vg_name

    TestObj.tok("lvcreate -L100m -V100m -T %s/pool1 -n lv1" % vg_name)
    TestObj.tok("lvcreate -i2 -L100m -V100m -T %s/pool2 -n lv2" % vg_name)

    for lv_num in range(1, 3):
        TestObj.tok("lvresize -f -l-2 %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name,
                                {"lv_size": "92.00m"})
        # Default unit is m
        TestObj.tok("lvresize -f -L-8 -n %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_size": "84.00m"})

        TestObj.tok("lvresize -f -L-8m -n %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_size": "76.00m"})

        # set specific size
        TestObj.tok("lvresize -f -l18 -n %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_size": "72.00m"})

        TestObj.tok("lvresize -f -L64m -n %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_size": "64.00m"})

        # Using test option so size will not change
        TestObj.tok("lvresize -f -l-1%%FREE --test %s/lv%d" % (vg_name, lv_num))
        TestObj.tok("lvresize -f -l-1%%PVS --test %s/lv%d" % (vg_name, lv_num))
        # Extending thin LV based on VG does not make sense, but leaving this for now
        TestObj.tok("lvresize -f -l-1%%VG -t %s/lv%d" % (vg_name, lv_num))
        TestObj.tok("lvresize -f -l-1%%VG -t %s/lv%d" % (vg_name, lv_num))

        if not linux.mkdir(lv_mnt):
            TestObj.tfail("Could not create %s" % lv_mnt)
            return False
        # xfs does not support reduce
        fs = "ext4"

        lv_device = "/dev/mapper/%s-lv%d" % (vg_name, lv_num)
        if not linux.mkfs(lv_device, fs, force=True):
            TestObj.tfail("Could not create fs on %s" % lv_device)
            return False

        if not linux.mount(lv_device, lv_mnt):
            TestObj.tfail("Could not mount %s" % lv_device)
            return False
        TestObj.tok("dd if=/dev/urandom of=%s/lv%d bs=1M count=5" % (lv_mnt, lv_num))

        # reduce FS
        TestObj.tok("yes 2>/dev/null | lvresize -rf -l-2 %s/lv%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "lv%d" % lv_num, vg_name, {"lv_size": "56.00m"})

        # snapshot
        snap_device = "/dev/mapper/%s-snap%d" % (vg_name, lv_num)
        TestObj.tok("lvcreate -K -s %s/lv%d -n snap%d" % (vg_name, lv_num, lv_num))
        if not linux.mkdir(snap_mnt):
            TestObj.tfail("Could not create %s" % snap_mnt)
            return False

        if not linux.mkfs(snap_device, fs, force=True):
            TestObj.tfail("Could not create fs on %s" % snap_device)
            return False

        if not linux.mount(snap_device, snap_mnt):
            TestObj.tfail("Could not mount %s" % snap_device)
            return False

        TestObj.tok("dd if=/dev/urandom of=%s/lv%d bs=1M count=5" % (snap_mnt, lv_num))
        # reduce FS
        TestObj.tok("yes 2>/dev/null | lvresize -l-2 -rf %s/snap%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "snap%d" % lv_num, vg_name, {"lv_size": "48.00m"})
        TestObj.trun("df -h %s" % snap_mnt)

        TestObj.tok("yes 2>/dev/null | lvresize -L40 -rf %s/snap%d" % (vg_name, lv_num))
        check_lv_expected_value(TestObj, "snap%d" % lv_num, vg_name, {"lv_size": "40.00m"})
        TestObj.trun("df -h %s" % snap_mnt)

        if not linux.umount(lv_mnt):
            TestObj.tfail("Could not umount %s" % lv_device)
            return False
        if not linux.umount(snap_mnt):
            TestObj.tfail("Could not umount %s" % snap_device)
            return False
    TestObj.tok("lvremove -ff %s" % vg_name)


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting Thin Resize test")
    print(80 * "#")

    linux.install_package("lvm2")

    # Create 4 devices
    for dev_num in range(1, 5):
        new_dev = loopdev.create_loopdev(size=256)
        if not new_dev:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False
        loop_dev["dev%s" % dev_num] = new_dev

    pvs = ""
    for dev in loop_dev.keys():
        pvs += " %s" % (loop_dev[dev])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    print(80 * "#")
    print("INFO: Starting extend test")
    print(80 * "#")
    _extend_pool()
    _extend_thin_lv()

    print(80 * "#")
    print("INFO: Starting reduce test")
    print(80 * "#")
    _reduce_pool()
    _reduce_thin_lv()

    return True


def _clean_up():
    linux.umount(lv_mnt)
    linux.umount(snap_mnt)

    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    try:
        start_test()
    except Exception as e:
        traceback.print_exc()
        TestObj.tfail("There was some problem while running the test (%s)" % e)
        print(e)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from time import sleep
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_var, write_var
from libsan.host.lvm import vg_remove


def cleanup_lvm():
    print("INFO: Cleaning up free disks to previous state.")
    errors = []

    print("Waiting for data stream to settle before logging out of iscsi.")
    sleep(5)

    vg_name = read_var("STRATIS_VG")
    atomic_run("Removing vg and all lvs",
               force=True,
               vg_name=vg_name,
               command=vg_remove,
               errors=errors
               )

    atomic_run("Cleaning var STRATIS_VG",
               command=clean_var,
               var="STRATIS_VG",
               errors=errors)

    atomic_run("Cleaning var STRATIS_DEVICE",
               command=clean_var,
               var="STRATIS_DEVICE",
               errors=errors)

    backup = read_var("STRATIS_DEVICE_BACKUP")
    if backup:
        atomic_run("Cleaning var STRATIS_DEVICE_BACKUP",
                   command=clean_var,
                   var="STRATIS_DEVICE_BACKUP",
                   errors=errors)

        atomic_run("Writing var STRATIS_DEVICE",
                   command=write_var,
                   var={'STRATIS_DEVICE': backup},
                   errors=errors)

    return errors


if __name__ == "__main__":
    errs = cleanup_lvm()
    exit(parse_ret(errs))



from __future__ import absolute_import, division, print_function, unicode_literals
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

"""LogChecker.py: Module to Check for errors on the system."""

__author__ = "Bruno Goncalves"
__copyright__ = "Copyright (c) 2015 Red Hat, Inc. All rights reserved."

import re  # regex
import os
import libsan.host.linux
from libsan.host.cmdline import run
import stqe.host.beaker

segfault_msg = " segfault "
calltrace_msg = "Call Trace:"

error_mgs = [segfault_msg, calltrace_msg]


def check_all():
    """Check for error on the system
    Returns:
    \tBoolean:
    \t\tTrue is no error was found
    \t\tFalse if some error was found
    """
    print("INFO: Checking for error on the system")
    error = 0

    if not kernel_check():
        error += 1
    if not abrt_check():
        error += 1
    if not messages_dump_check():
        error += 1
    if not dmesg_check():
        error += 1
    if not console_log_check():
        error += 1
    if not kdump_check():
        error += 1

    if error:
        log_messages = "/var/log/messages"
        if os.path.isfile(log_messages):
            print("submit %s, named messages.log" % log_messages)
            run("cp %s messages.log" % log_messages)
            stqe.host.beaker.log_submit("messages.log")

        print("INFO: Umounting NFS to avoid sosreport being hang there")
        run("umount /var/crash")

        ret_code = run("which sosreport", verbose=False)
        if ret_code != 0:
            print("WARN: sosreport is not installed")
            print("INFO: Mounting NFS again")
            run("mount /var/crash")
            return False

        print("Generating sosreport log")
        disable_plugin = ""
        # Run sosreport without EMC plugin, it has a problem when NaviCLI is installed BZ888715
        # check if sosreport supports EMC plugin
        if run("sosreport --list-plugins | grep -w emc") == 0:
            disable_plugin = "-n emc"
        # sosreport command is deprecated in new versions... If we run into some problems with it
        # we should change it to 'sos report'
        ret_code, sosreport_log = run("timeout 1h sosreport --batch %s" % disable_plugin, return_output=True)
        if ret_code != 0:
            print("FAIL: sosreport command failed")
            print("INFO: Mounting NFS again")
            run("mount /var/crash")
            return False

        sos_lines = sosreport_log.split("\n")
        sos_file = None
        for line in sos_lines:
            # In RHEL7 sosreport is saving under /var/tmp while RHEL6 uses /tmp...
            m = re.match(r"\s+((\/var)?\/tmp\/sosreport\S+)", line)
            if m:
                sos_file = m.group(1)
        if not sos_file:
            print("WARN: could not save sosreport log")
            print("INFO: Mounting NFS again")
            run("mount /var/crash")
            return False

        stqe.host.beaker.log_submit(sos_file)
        print("INFO: Mounting NFS again")
        run("mount /var/crash")

        return False
    return True


def abrt_check():
    """Check if abrtd found any issue
    Returns:
    \tBoolean:
    \t\tTrue no error was found
    \t\tFalse some error was found
    """
    print("INFO: Checking abrt for error")

    if run("rpm -q abrt", verbose=False) != 0:
        print("WARN: abrt tool does not seem to be installed")
        print("WARN: skipping abrt check")
        return True

    if run("pidof abrtd", verbose=False) != 0:
        print("FAIL: abrtd is not running")
        return False

    ret, log = run("abrt-cli list", return_output=True)
    if ret != 0:
        print("FAIL: abrt-cli command failed")
        return False

    # We try to match for "Directory" to check if
    # abrt-cli list is actually listing any issue
    error = False
    if log:
        lines = log.split("\n")
        for line in lines:
            m = re.match(r"Directory:\s+(\S+)", line)
            if m:
                directory = m.group(1)
                filename = directory
                filename = filename.replace(":", "-")
                filename += ".tar.gz"
                run("tar cfzP %s %s" % (filename, directory))
                stqe.host.beaker.log_submit(filename)
                # if log is saved on beaker, it can be deleted from server
                # it avoids next test from detecting this failure
                run("abrt-cli rm %s" % directory)
                error = True

    if error:
        print("FAIL: Found abrt error")
        return False

    print("PASS: no Abrt entry has been found.")
    return True


def kernel_check():
    """
    Check if kernel got tainted.
    It checks /proc/sys/kernel/tainted which returns a bitmask.
    The values are defined in the kernel source file include/linux/kernel.h,
    and explained in kernel/panic.c
    cd /usr/src/kernels/`uname -r`/
    Sources are provided by kernel-devel
    Returns:
    \tBoolean:
    \t\tTrue if did not find any issue
    \t\tFalse if found some issue
    """
    print("INFO: Checking for tainted kernel")

    previous_tainted_file = "/tmp/previous-tainted"

    ret, tainted = run("cat /proc/sys/kernel/tainted", return_output=True)

    tainted_val = int(tainted)
    if tainted_val == 0:
        run("echo %d > %s" % (tainted_val, previous_tainted_file), verbose=False)
        print("PASS: Kernel is not tainted.")
        return True

    print("WARN: Kernel is tainted!")

    if not os.path.isfile(previous_tainted_file):
        run("echo %s > %s" % (tainted_val, previous_tainted_file), verbose=False)
    ret, prev_taint = run("cat %s" % previous_tainted_file, return_output=True)
    prev_taint_val = int(prev_taint)
    if prev_taint_val == tainted_val:
        print("INFO: Kernel tainted has already been handled")
        return True

    run("echo %d > %s" % (tainted_val, previous_tainted_file), verbose=False)

    # check all bits that are set
    bit = 0
    while tainted_val != 0:
        # need to change back to int because it got changed to float at shifting
        tainted_val = int(tainted_val)
        if tainted_val & 1:
            print("\tTAINT bit %d is set\n" % bit)
        bit += 1
        # shift tainted value
        tainted_val /= 2
    # List all tainted bits that are defined
    print("List bit definition for tainted kernel")
    run("cat /usr/src/kernels/`uname -r`/include/linux/kernel.h | grep TAINT_")

    found_issue = False
    # try to find the module which tainted the kernel, tainted module have a mark between '('')'
    ret, output = run("cat /proc/modules | grep -e '(.*)' |  cut -d' ' -f1", return_output=True)
    tainted_mods = output.split("\n")
    # For example during iscsi async_events scst tool loads an unsigned module
    # just ignores it, so we will ignore this tainted if there is no tainted
    # modules loaded
    if not tainted_mods:
        print("INFO: ignoring tainted as the module is not loaded anymore")
    else:
        # ignore ocrdma due BZ#1334675
        # ignore nvme_fc and nvmet_fc due Tech Preview - BZ#1384922
        ignore_modules = ["ocrdma", "nvme_fc", "nvmet_fc"]
        for tainted_mod in tainted_mods:
            if tainted_mod:
                print("INFO: The following module got tainted: %s" % tainted_mod)
                run("modinfo %s" % tainted_mod)
                # due BZ#1334675  we are ignoring ocrdma module
                if tainted_mod in ignore_modules:
                    print("INFO: ignoring tainted on %s" % tainted_mod)
                    run("echo %d > %s" % (tainted_val, previous_tainted_file), verbose=False)
                    continue
                found_issue = True

    run("echo %s > %s" % (tainted, previous_tainted_file), verbose=False)
    if found_issue:
        return False

    return True


def _date2num(date):
    date_map = {"Jan": "1",
                "Feb": "2",
                "Mar": "3",
                "Apr": "4",
                "May": "5",
                "Jun": "6",
                "Jul": "7",
                "Aug": "8",
                "Sep": "9",
                "Oct": "10",
                "Nov": "11",
                "Dec": "12"}

    date_regex = r"(\S\S\S)\s(\d+)\s(\d\d:\d\d:\d\d)"
    m = re.match(date_regex, date)
    month = date_map[m.group(1)]
    day = str(m.group(2))
    # if day is a single digit, add '0' to begin
    if len(day) == 1:
        day = "0" + day

    hour = m.group(3)
    hour = hour.replace(":", "")

    value = month + day + hour

    return value


def messages_dump_check():
    previous_time_file = "/tmp/previous-dump-check"

    log_msg_file = "/var/log/messages"
    if not os.path.isfile(log_msg_file):
        print("WARN: Could not open %s" % log_msg_file)
        return True

    log_file = open(log_msg_file, 'rb')
    log = ""
    for line in log_file.readlines():
        try:
            log += line.decode("UTF-8")
        except UnicodeDecodeError:
            log += line.decode("latin-1")

    begin_tag = "\\[ cut here \\]"
    end_tag = "\\[ end trace "

    if not os.path.isfile(previous_time_file):
        first_time = "Jan 01 00:00:00"
        time = _date2num(first_time)
        run("echo %s > %s" % (time, previous_time_file))

    # Read the last time test ran
    ret, last_run = run("cat %s" % previous_time_file, return_output=True)
    print("INFO: Checking for stack dump messages after: %s" % last_run)

    # Going to search the file for text that matches begin_tag until end_tag
    dump_regex = begin_tag + "(.*?)" + end_tag
    m = re.findall(dump_regex, log, re.MULTILINE)
    if m:
        print("INFO: Checking if it is newer than: %s" % last_run)
        print(m.group(1))
        # TODO

    print("PASS: No recent dump messages has been found.")
    return True


def dmesg_check():
    """
    Check for error messages on dmesg ("Call Trace and segfault")
    """
    print("INFO: Checking for errors on dmesg.")
    error = 0
    for msg in error_mgs:
        ret, output = run("dmesg | grep -i '%s'" % msg, return_output=True)
        if output:
            print("FAIL: found %s on dmesg" % msg)
            run("echo '\nINFO found %s  Saving it\n'>> dmesg.log" % msg)
            run("dmesg >> dmesg.log")
            stqe.host.beaker.log_submit("dmesg.log")
            error = 1
    libsan.host.linux.clear_dmesg()
    if error:
        return False

    print("PASS: No errors on dmesg have been found.")
    return True


def console_log_check():
    """
    Checks for error messages on console log ("Call Trace and segfault")
    """
    error = 0
    console_log_file = "/root/console.log"
    prev_console_log_file = "/root/console.log.prev"
    new_console_log_file = "/root/console.log.new"

    if not os.environ.get('LAB_CONTROLLER'):
        print("WARN: Could not find lab controller")
        return True

    if not os.environ.get('RECIPEID'):
        print("WARN: Could not find recipe ID")
        return True

    lab_controller = os.environ['LAB_CONTROLLER']
    recipe_id = os.environ['RECIPEID']

    # get current console log
    url = "http://%s:8000/recipes/%s/logs/console.log" % (lab_controller, recipe_id)

    if run("wget -q %s -O %s" % (url, new_console_log_file)) != 0:
        print("INFO: Could not get console log")
        # return sucess if could not get console.log
        return True

    # if there was previous console log, we just check the new part
    run("diff -N -n --unidirectional-new-file %s %s > %s" % (
        prev_console_log_file, new_console_log_file, console_log_file))

    # backup the current full console.log
    # next time we run the test we will compare just
    # what has been appended to console.log
    run("mv -f %s %s" % (new_console_log_file, prev_console_log_file))

    print("INFO: Checking for errors on %s" % console_log_file)
    for msg in error_mgs:
        ret, output = run("cat %s | grep -i '%s'" % (console_log_file, msg), return_output=True)
        if output:
            print("INFO found %s on %s" % (msg, console_log_file))
            stqe.host.beaker.log_submit(console_log_file)
            error = + 1

    if error:
        return False

    print("PASS: No errors on %s have been found." % console_log_file)
    return True


def kdump_check():
    """
    Check for kdump error messages.
    It assumes kdump is configured on /var/crash
    """
    error = 0

    previous_kdump_check_file = "/tmp/previous-kdump-check"
    kdump_dir = "/var/crash"
    ret, hostname = run("hostname", verbose=False, return_output=True)

    if not os.path.exists("%s/%s" % (kdump_dir, hostname)):
        print("INFO: No kdump log found for this server")
        return True

    ret, output = run("ls -l %s/%s |  awk '{print$9}'" % (kdump_dir, hostname), return_output=True)
    kdumps = output.split("\n")
    kdump_dates = []
    for kdump in kdumps:
        if kdump == "":
            continue
        # parse on the date, remove the ip of the uploader
        m = re.match(".*?-(.*)", kdump)
        if not m:
            print("WARN: unexpected format for kdump (%s)" % kdump)
            continue
        date = m.group(1)
        # Old dump were using "."
        date = date.replace(r"\.", "-")
        # replace last "-" with space to format date properly
        index = date.rfind("-")
        date = date[:index] + " " + date[index + 1:]
        print("INFO: Found kdump from %s" % date)
        kdump_dates.append(date)

    # checks if a file to store last run exists, if not create it
    if not os.path.isfile("%s" % previous_kdump_check_file):
        # time in seconds
        ret, time = run(r"date +\"\%s\"", verbose=False, return_output=True)
        run("echo -n %s > %s" % (time, previous_kdump_check_file), verbose=False)
        print("INFO: kdump check is executing for the first time.")
        print("INFO: doesn't know from which date should check files.")
        print("PASS: Returning success.")
        return True

    # Read the last time test ran
    ret, previous_check_time = run("cat %s" % previous_kdump_check_file, return_output=True)
    # just add new line to terminal because the file should not have already new line character
    print("")

    for date in kdump_dates:
        # Note %% is escape form to use % in a string
        ret, kdump_time = run("date --date=\"%s\" +%%s" % date, return_output=True)
        if ret != 0:
            print("WARN: Could not convert date %s" % date)
            continue

        if not kdump_time:
            continue
        if int(kdump_time) > int(previous_check_time):
            print("FAIL: Found a kdump log from %s (more recent than %s)" % (date, previous_check_time))
            print("FAIL: Check %s/%s" % (kdump_dir, hostname))
            error += 1

    ret, time = run(r"date +\"\%s\"", verbose=False, return_output=True)
    run("echo -n %s > %s" % (time, previous_kdump_check_file), verbose=False)

    if error:
        return False

    print("PASS: No errors on kdump have been found.")
    return True



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var


def cleanup_configure():
    print("INFO: Cleaning up libstoragemgmt configurations.")
    errors = []

    ret, variables = atomic_run("Getting variables to clean.",
                                command=run,
                                cmd="ls -la /tmp",
                                return_output=True,
                                errors=errors)

    variables = [value for line in variables.splitlines() for value in line.split() if value.startswith("LSM_")]
    print("Will clean these variables: %s" % " ".join(variables))

    for var in variables:
        atomic_run("Cleaning var %s" % var,
                   command=clean_var,
                   var=var,
                   errors=errors)

    return errors


if __name__ == "__main__":
    errs = cleanup_configure()
    exit(parse_ret(errs))



#!/usr/bin/python -u

# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
import libsan.host.scsi as scsi
import libsan.host.mp as mp
import libsan.host.linux as linux
import libsan.host.dt as dt
import libsan.sanmgmt as sanmgmt
import stqe.host.nfs_lock as nfs_lock
import stqe.host.LogChecker as log
import re
import sys
import subprocess
import argparse
from random import randint

LOCK_TYPE = "shared"

clean_up_info = {}
lock_obj = None


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)

    # Append time information to command
    date = "date \"+%Y-%m-%d %H:%M:%S\""
    p = subprocess.Popen(date, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    stdout, stderr = p.communicate()
    stdout = stdout.decode('ascii', 'ignore')
    stdout = stdout.rstrip("\n")

    print("[%s] %s" % (stdout, string))

    sys.stdout.flush()
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def _clean_up():
    """
    """
    global LOCK_TYPE, clean_up_info, lock_obj

    if not clean_up_info:
        return True

    scsi_disks = clean_up_info["scsi_disks"]
    if scsi_disks:
        for scsi_disk in scsi_disks:
            scsi.disk_sys_trigger(scsi_disk, "UP")

    if lock_obj:
        lock_obj.release_lock(LOCK_TYPE)
    _print("INFO: Clean up done.")
    return True


def _check_can_bring_disk_down(scsi_disk):
    """
    Before starting the test, make sure disk can be brought offline
    """
    if not scsi.disk_sys_trigger(scsi_disk, "DOWN"):
        _print("FAIL: Failed to bring disk offline")
        return False
    # make sure we are not able to write to the disk
    ret = run("dd if=/dev/%s of=/dev/null count=1 bs=512 iflag=direct" % scsi_disk)
    if ret == 0:
        _print("FAIL: I/O can still goes to /dev/%s even sysfs marked it as offline"
               % scsi_disk)
        return False
    _print("INFO: Disk is offline as expected")
    if not scsi.disk_sys_trigger(scsi_disk, "UP"):
        _print("FAIL: Failed to bring disk online")
        return False

    return True


def execute_test(mpath_name, rounds, max_interval):
    global LOCK_TYPE, clean_up_info, lock_obj

    mp_info = mp.multipath_query_all(mpath_name)
    if not mp_info:
        _print("FAIL: Got NULL from query_mp_info(%s)" % mpath_name)
        return False

    obj_sanmgmt = sanmgmt.create_sanmgmt_for_mpath(mpath_name)
    if not obj_sanmgmt:
        _print("FAIL: Got NULL from create_sanmgmt_for_mpath(%s)" % mpath_name)
        return False

    clean_up_info["obj_sanmgmt"] = obj_sanmgmt

    scsi_disks = list(mp_info["disk"].keys())
    clean_up_info["scsi_disks"] = scsi_disks

    lock_obj = nfs_lock.setup_nfs_lock_for_mpath(mpath_name)
    if lock_obj:
        _print("INFO: Requesting NFS lock (%s)..." % LOCK_TYPE)
        if not lock_obj.request_lock(LOCK_TYPE):
            _print("FAIL: Could not request NFS lock")
            return False

        _print("INFO: Waiting for NFS lock...")
        if not lock_obj.get_lock():
            _print("FAIL: Give up waiting for lock")
            return False
        _print("INFO: Good, got NFS lock. Start testing...")

    # Just write a bit to make sure there will be an active disk
    if run("dd if=/dev/urandom of=/dev/mapper/%s bs=1k count=256" % mpath_name) != 0:
        mp.multipath_show()
        _print("FAIL: Could not perform IO on %s" % mpath_name)
        return False

    active_disks = mp.mpath_get_active_disk(mpath_name)
    if not active_disks:
        mp.multipath_show()
        _print("FAIL: Could not find any active disk for %s" % mpath_name)
        return False

    for active_disk in active_disks:
        _print("INFO: Testing if it is possible to bring the scsi disk down")
        if not _check_can_bring_disk_down(active_disk):
            _print("FAIL: fail bringing disk %s down" % active_disk)
            return False

    dt.dt_init_data(of="/dev/mapper/%s" % mpath_name)

    _print("INFO: Will trigger sysfs offline/online for these disks of %s" % mpath_name)
    for scsi_disk in scsi_disks:
        print("%8s%s" % (" ", scsi_disk))

    _print("INFO: Bouncing count is %d with max interval of %d" % (rounds, max_interval))
    counter = 0
    while rounds > counter:
        counter += 1
        _print("INFO: Round %d" % counter)
        for scsi_disk in scsi_disks:
            for active_disk in active_disks:
                if not scsi.disk_sys_trigger(scsi_disk, "DOWN"):
                    _print("FAIL: Could not bring disk %s DOWN" % active_disk)
                    return False
            interval = randint(1, max_interval)
            _print("INFO: %s offline interval %d" % (scsi_disk, interval))
            linux.sleep(interval)

            for active_disk in active_disks:
                if not scsi.disk_sys_trigger(scsi_disk, "UP"):
                    _print("FAIL: Could not bring disk %s UP" % active_disk)
                    return False
            interval = randint(1, max_interval)
            _print("INFO: %s online interval %d" % (scsi_disk, interval))
            linux.sleep(interval)

    linux.wait_udev()
    mp.multipath_reload()
    _print("INFO: Bouncing done for %d rounds." % rounds)

    _print("INFO: Verify data on mpath: %s" % mpath_name)
    dt.dt_verify_data("/dev/mapper/%s" % mpath_name)

    _print("INFO: Data verification PASS on %s" % mpath_name)

    _print("INFO: Running I/O stress on %s" % mpath_name)
    if not dt.dt_stress("/dev/mapper/%s" % mpath_name):
        _print("FAIL: DT I/O stress failed on %s" % mpath_name)
        return False

    _print("INFO: Online/Offline test PASS")
    if not log.check_all():
        _print("FAIL: detected error on LogChecker")
        return False

    _print("PASS: Online/Offline test")
    return True


def main():
    """
    """
    PASS = 0
    FAIL = 1

    parser = argparse.ArgumentParser(
        description='add new num')
    parser.add_argument("--mpath-name", "-m", required=False, dest="mpath_name",
                        help="Name of multipath device to use", metavar="mpath")
    parser.add_argument("--rounds", required=False, dest="rounds", default=100,
                        help="How many times the disks will be tests", type=int, metavar="number")
    parser.add_argument("--max-interval", required=False, dest="max_interval", type=int,
                        default=10, help="Maximum time in seconds to wait with disk online/offline", metavar="time")

    args = parser.parse_args()

    mpath_name_list = []
    if args.mpath_name:
        mpath_name_list.append(args.mpath_name)
    rounds = args.rounds
    max_interval = args.max_interval

    # If not mpath was specified search for them on the system
    if not mpath_name_list:
        choosen_devs = sanmgmt.choose_mpaths()
        if not choosen_devs:
            _print("FAIL: Could not find any multipath device to use")
            sys.exit(FAIL)
        mpath_name_list = list(choosen_devs.keys())

    error = 0
    for mpath_name in mpath_name_list:
        if not execute_test(mpath_name, rounds, max_interval):
            error += 1
        _clean_up()

    if error:
        sys.exit(FAIL)
    sys.exit(PASS)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.stratis import Stratis
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, read_env


def bind_pool():
    errors = []
    key_desc = ''
    trust_url = None
    binding_method = None
    tang_url = None
    thumbprint = None
    stratis = Stratis()

    pool_name = read_var(read_env("fmf_pool_name"))
    try:
        key_desc = read_env('fmf_key_desc')
    except KeyError:
        pass
    try:
        trust_url = read_env("fmf_trust_url")
    except KeyError:
        pass
    try:
        thumbprint = read_var(read_env("fmf_thumbprint"))
    except KeyError:
        pass
    try:
        binding_method = read_env("fmf_binding_method")
    except KeyError:
        pass
    try:
        tang_url = read_var(read_env("fmf_tang_url"))
    except KeyError:
        pass

    atomic_run("Creating stratis pool %s." % pool_name,
               command=stratis.pool_bind,
               pool_name=pool_name,
               key_desc=key_desc,
               binding_method=binding_method,
               thumbprint=thumbprint,
               tang_url=tang_url,
               trust_url=trust_url,
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = bind_pool()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import write_var, read_env
from libsan.host.cmdline import run
from libsan.host.linux import service_start, is_service_running, service_stop, install_package, hostname


def setup_tang():
    errors = []

    cleanup = None
    tang_service_name = "tangd.socket"

    install_package("tang")
    install_package("curl")
    install_package('jose')
    install_package('jq')
    install_package("coreutils")

    try:
        cleanup = read_env("fmf_tang_cleanup")
    except KeyError:
        pass

    if not cleanup:
        atomic_run("Start tang server",
                   command=service_start,
                   service_name=tang_service_name,
                   errors=errors
                   )

        if is_service_running("firewalld"):
            atomic_run("Stopping firewalld service",
                       command=service_stop,
                       service_name="firewalld",
                       errors=errors
                       )

        ret, data = atomic_run("Getting Tang thumbprint of server",
                               return_output=True,
                               command=run,
                               cmd=f'curl -s {hostname()}/adv | jq -r .payload | base64 -d |'
                                   f' jose jwk use -i- -r -u verify -o- | jose jwk thp -i- ',
                               errors=errors
                               )

        atomic_run("Writing var TANG_THUMBPRINT",
                   command=write_var,
                   var={'TANG_THUMBPRINT': data},
                   errors=errors)

        atomic_run("Writing var TANG_URL",
                   command=write_var,
                   var={'TANG_URL': f"http://{hostname()}"},
                   errors=errors)

        return errors

    atomic_run("Stopping tangd.socket",
               service_name=tang_service_name,
               command=service_stop,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = setup_tang()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.scsi import get_free_disks
from stqe.host.persistent_vars import write_var, read_var
from libsan.host.linux import is_service_running
from libsan.host.lvm import vg_create, lv_create


def setup_lvm():
    msg = "INFO: Getting local disks."
    print(msg)
    vg_name = "stratis_vg"
    lv_name = "stratis_lv_"
    errors = []

    disks = atomic_run(message="Getting free disks",
                       command=get_free_disks,
                       errors=errors)
    if disks is None:
        msg = "FAIL: Could not find any free disks."
        print(msg)
        errors.append(msg)
        return errors

    disks = disks.keys()
    disk_paths = ["/dev/" + j for j in disks]
    blockdevs = read_var("STRATIS_DEVICE")

    if blockdevs:
        # backup the previous devices
        atomic_run("Writing var STRATIS_DEVICE_BACKUP",
                   command=write_var,
                   var={'STRATIS_DEVICE_BACKUP': " ".join(blockdevs)},
                   errors=errors)
        if not isinstance(blockdevs, list):
            blockdevs = [blockdevs]
        disk_paths += [x for x in blockdevs if x not in blockdevs]

    print("Using these blockdevs: %s" % " ".join(disk_paths))
    for disk in disk_paths:
        atomic_run("Zeroing superblock of disk %s." % disk,
                   command=run,
                   cmd="dd if=/dev/zero of=%s bs=1M count=10" % disk,
                   errors=errors)
        if is_service_running("multipathd"):
            atomic_run("remove multipath superblock of disk %s." % disk,
                       command=run,
                       cmd="multipath -W %s" % disk,
                       errors=errors)

    atomic_run("Creating volume group from available disks",
               pv_name=" ".join(disk_paths),
               vg_name=vg_name,
               command=vg_create,
               errors=errors
               )

    lv_paths = []

    for i in range(4):
        atomic_run(f"Creating logical volume with_name {lv_name}{i}",
                   options=["-l 25%VG"],
                   vg_name=vg_name,
                   lv_name=f"{lv_name}{i}",
                   command=lv_create,
                   errors=errors
                   )

        lv_paths.append(f"/dev/mapper/{vg_name}-{lv_name}{i}")

    print("INFO: Will use following paths:" + " \n".join(lv_paths))

    atomic_run("Writing var STRATIS_VG",
               command=write_var,
               var={'STRATIS_VG': vg_name},
               errors=errors)

    atomic_run("Writing var STRATIS_AVAILABLE_DEVICES",
               command=write_var,
               var={'STRATIS_AVAILABLE_DEVICES': lv_paths},
               errors=errors)

    atomic_run("Writing var STRATIS_DEVICE",
               command=write_var,
               var={'STRATIS_DEVICE': lv_paths},
               errors=errors)

    atomic_run(message="Listing block devices.",
               command=run,
               cmd="lsblk",
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = setup_lvm()
    exit(parse_ret(errs))



from __future__ import absolute_import, division, print_function, unicode_literals
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

"""iscsi.py: Module with test specific method for iSCSI."""

__author__ = "Bruno Goncalves"
__copyright__ = "Copyright (c) 2016 Red Hat, Inc. All rights reserved."

import re  # regex
import libsan.host.iscsi as iscsi
import libsan.host.linux as linux
import libsan.sanmgmt as sanmgmt


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)
    print(string)
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def get_iscsi_conf_host_ifaces():
    """
    Check if server has iSCSI interfaces configured
    If there is no configuration, it should use the default
    Returns:
    \tlist of iSCSI interface information
    """
    obj_sanmgmt = sanmgmt.SanMgmt()
    host_conf = obj_sanmgmt.iscsi_host_conf()
    if not host_conf:
        return None

    if "ifaces" not in host_conf:
        return None

    configured_ifaces = []
    for iface in host_conf["ifaces"].keys():
        if "name" not in host_conf["ifaces"][iface]:
            _print("FAIL: get_iscsi_conf_host_ifaces()- interface defined without name")
            print(host_conf["ifaces"][iface])
            return None
        configured_ifaces.append(host_conf["ifaces"][iface]["name"])
    return configured_ifaces


def get_iscsi_conf_host_portals():
    """
    Check if server has iSCSI interfaces configured
    If there is no configuration, it should use the default
    Returns:
    \tlist of iSCSI interface information
    """
    obj_sanmgmt = sanmgmt.SanMgmt()
    host_conf = obj_sanmgmt.iscsi_host_conf()
    if not host_conf:
        return None

    if "ifaces" not in host_conf:
        return None

    configured_portals = []
    for iface in host_conf["ifaces"].keys():
        iface_info = host_conf["ifaces"][iface]
        if "target_ip" in iface_info and iface_info["target_ip"] not in configured_portals:
            configured_portals.append(iface_info["target_ip"])
    return configured_portals


def get_host_ifaces():
    """
    Return host configured interfaces names. Configures them if needed
    If iSCSI boot will create new interfaces to be used
    """
    # server does not have any interface configured, use default one
    conf_ifaces = get_iscsi_conf_host_ifaces()
    if not conf_ifaces:
        return ["default"]

    is_iscsi_boot = iscsi.is_iscsi_boot()

    all_ifaces = iscsi.get_iscsi_iface_names()

    host_ifaces = []
    for iface in conf_ifaces:
        iface_name = iface["interface_name"]
        # We should not use the iSCSI boot interface for test
        # Exception is qla4xxx interfaces
        if is_iscsi_boot and not re.match("qla4xxx.*", iface_name):
            new_iface = "clone_%s" % iface_name
            if new_iface not in all_ifaces():
                _print("WARN: TODO - Need to clone the interface, and not just create new")
                if not iscsi.create_iscsi_iface(new_iface):
                    _print("FAIL: Could not create %s iface" % new_iface)
                    return None
            iface_name = new_iface

        iface_ip = None
        if "ipv4_address" in iface:
            iface_ip = iface["ipv4_address"]
        # If not IP is set on bnx2i driver we need to enable DHCP
        if re.match("bnx2i.*", iface_name) and not iface_ip:
            iface_ip = "0.0.0.0"

        if iface_ip:
            if not iscsi.iface_set_ip(iface_name, iface_ip):
                _print("FAIL: Could not set IP for iface: %s" % iface_name)
                return None
        host_ifaces.append(iface_name)

    return host_ifaces


def get_host_portals():
    """
    Return a list of host configured portals. If none is set try to use array based on host location
    """
    # server does not have any interface configured, use default one
    conf_portals = get_iscsi_conf_host_portals()
    if conf_portals:
        return conf_portals

    san_conf = sanmgmt.SanMgmt()
    san_conf.load_conf()
    host = linux.hostname()
    if "brq" in host:
        return [san_conf.san_conf_dict["alias"]["ip_brq_equ"]]
    # Use bos array as default
    return [san_conf.san_conf_dict["alias"]["boston_array"]]


def auto_conf(test_type='io_stress'):
    """
    Usage
        auto_conf('test_type')
    Purpose
        Configure iSCSI on generic hosts (not in san_top.conf)
    Parameter
        test_type
    Returns
        scsi_ids   # the iSCSI disks found
            or
        None
    """

    obj_sanmgmt = sanmgmt.SanMgmt()
    host_conf = obj_sanmgmt.iscsi_host_conf()

    linux.install_package("iscsi-initiator-utils")
    iscsi.clean_up()

    iqn_base = 'iqn.1994-05.com.redhat:'
    iqn_tail = None
    arch = linux.os_arch()
    arch = arch.replace('_', '-')

    if test_type == 'io_stress':
        iqn_tail = 'io-stress-tcp-' + arch
        iscsi.set_iscsid_parameter(**{'node.session.cmds_max': '4096', 'node.session.queue_depth': '128'})

    if test_type == 'chap-1way':
        iqn_tail = 'chap-1way-' + arch
        iscsi.set_chap('redhat_out_user', 'redhat_out_pass')

    if test_type == 'chap-2ways':
        iqn_tail = 'chap-2ways-' + arch
        iscsi.set_chap('redhat_out_user', 'redhat_out_pass', 'redhat_in_user', 'redhat_in_pass')

    if test_type == 'big-lun':
        iqn_tail = 'big-lun-' + arch

    if test_type == 'mdadm':
        iqn_tail = 'mdadm-' + arch

    if test_type == 'discard':
        iqn_tail = 'discard-' + arch

    if test_type == 'tcp-general':
        iqn_tail = 'tcp-general-' + arch

    if test_type == 'multipath-io':
        iqn_tail = 'multipath-io-' + arch

    if test_type == 'discovery-many-luns':
        iqn_tail = 'max'

    if test_type == 'os-reboot':
        iqn_tail = 'os-reboot'

    if test_type == 'lsm-test':
        iqn_tail = 'lsm-test-iqn'

    if test_type == 'vdo-general':
        iqn_tail = 'vdo-general'

    if test_type == 'vdo-small':
        iqn_tail = 'vdo-small'

    if test_type == 'stratis-general':
        iqn_tail = 'stratis-general'

    if iqn_tail is None:
        print('FAIL: setup_iscsi(): Wrong test_type')
        return False

    iqn = iqn_base + iqn_tail

    # Lowering iscsid timeout for sessions using multipath
    iscsi.multipath_timeo()

    iface_name = 'default'
    portals = get_host_portals()
    portal = portals[0]

    if host_conf is not None:
        for iface in host_conf["ifaces"].keys():
            iface_ip = None
            subnet_mask = None
            gateway = None

            if "name" in host_conf["ifaces"][iface]:
                iface_name = host_conf["ifaces"][iface]["name"]
            if "target_ip" in host_conf["ifaces"][iface]:
                portal = host_conf["ifaces"][iface]["target_ip"]
            if "ip" in host_conf["ifaces"][iface]:
                iface_ip = host_conf["ifaces"][iface]["ip"]
            if "mask" in host_conf["ifaces"][iface]:
                subnet_mask = host_conf["ifaces"][iface]["mask"]
            if "gateway" in host_conf["ifaces"][iface]:
                gateway = host_conf["ifaces"][iface]["gateway"]
            if test_type == 'io_stress':
                if "iqn" in host_conf["ifaces"][iface]:
                    iqn = host_conf["ifaces"][iface]["iqn"]

            if not discovery_login(iface_name=iface_name, portal=portal, iface_ip=iface_ip, subnet_mask=subnet_mask,
                                   gateway=gateway, iqn=iqn):
                return False

    else:
        if not discovery_login(iface_name=iface_name, portal=portal, iqn=iqn):
            return False

    return True


def discovery_login(iface_name, portal, iqn, iface_ip=None, subnet_mask=None, gateway=None):
    if not iface_name or not portal or not iqn:
        print('FAIL: auto_conf() - Missing iface_name, portal or iqn')
        return False

    if iface_ip:
        if not iscsi.iface_set_ip(iface_name, iface_ip, subnet_mask, gateway):
            _print("FAIL: auto_conf() - Could not set IP for %s" % iface_name)
            return False

    print('INFO: IQN will be set to ' + iqn)

    if not iscsi.iface_set_iqn(iqn, iface_name):
        _print("FAIL: auto_conf() - Could not set %s to iface %s" % (iqn, iface_name))
        return False

    if not iscsi.discovery_st(portal, ifaces=iface_name, disc_db=True):
        print("FAIL: auto_conf() - Could not discover any target on %s using iface %s" % (portal, iface_name))
        return False

    if not iscsi.node_login():
        _print("FAIL: auto_conf() - Could not login to new discovered portal")
        return False
    print("INFO: Iface %s logged in successfully to %s" % (iface_name, portal))

    return True



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.lsm import yield_lsm_config
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var
from os import environ


def create_fs():
    print("INFO: Creating FS.")
    errors = []

    var_fs_id = "LSM_" + environ["fmf_fs_id"]
    fs_id = read_var(var_fs_id)
    clone_name = environ["fmf_fs_cloned_name"]

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])

    ret, data = atomic_run("Cloning FS %s to FS %s." % (fs_id, clone_name),
                           command=lsm.fs_clone,
                           src_fs=fs_id,
                           dst_name=clone_name,
                           return_output=True,
                           errors=errors)
    cloned_fs_id = [line.split()[0].strip() for line in data.splitlines() if clone_name in line][0]

    atomic_run("Writing var LSM_FS_CLONED_ID",
               command=write_var,
               var={"LSM_FS_CLONED_ID": cloned_fs_id},
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = create_fs()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, clean_var


def cleanup_raid():
    raid = read_var('STRATIS_DEVICE')
    print("INFO: Cleaning up raid %s" % raid)
    errors = []

    atomic_run("Stopping raid %s" % raid,
               cmd="mdadm --stop %s" % raid,
               command=run,
               errors=errors)

    disks = read_var('RAID_DISKS')
    atomic_run("Zeroing superblock on disks: %s" % disks,
               cmd="mdadm --zero-superblock %s" % disks,
               command=run,
               errors=errors)

    atomic_run("Cleaning var STRATIS_DEVICE",
               command=clean_var,
               var="STRATIS_DEVICE",
               errors=errors)

    atomic_run("Cleaning var RAID_DISKS",
               command=clean_var,
               var="RAID_DISKS",
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = cleanup_raid()
    exit(parse_ret(errs))



#!/usr/bin/python -u

# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

#
# Bugs related:
#   BZ1344381 - multipath: IO does not fail when LUN is deleted on target
#   BZ1181962 - Call trace when running IO on multipath device that fails
#

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
import libsan.host.scsi as scsi
import libsan.host.mp as mp
import libsan.host.linux as linux
import libsan.host.dt as dt
import libsan.misc.time as time
import libsan.sanmgmt as sanmgmt
import stqe.host.nfs_lock as nfs_lock
import stqe.host.LogChecker as log
import re
import sys
import os
import subprocess
import argparse
import traceback

LOCK_TYPE = "shared"

clean_up_info = {}

lock_obj = None


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)

    # Append time information to command
    date = "date \"+%Y-%m-%d %H:%M:%S\""
    p = subprocess.Popen(date, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    stdout, stderr = p.communicate()
    stdout = stdout.decode('ascii', 'ignore')
    stdout = stdout.rstrip("\n")

    print("[%s] %s" % (stdout, string))

    sys.stdout.flush()
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def _clean_up():
    """
    """
    global LOCK_TYPE, clean_up_info

    if not clean_up_info:
        return True

    if clean_up_info["mpath_name"]:
        mp.remove_mpath(clean_up_info["mpath_name"])
        clean_up_info["mpath_name"] = None

    wwid = clean_up_info["wwid"]
    if wwid:
        linux.remove_device_wwid(wwid)
        clean_up_info["wwid"] = None
        _print("INFO: All disks removed from the server")

    obj_sanmgmt = clean_up_info["obj_sanmgmt"]
    lun_name = clean_up_info["lun_name"]
    if lun_name:
        _print("INFO: Deleting %s on the array..." % lun_name)
        if not obj_sanmgmt.lun_remove(lun_name):
            _print("WARN: Could not remove TMP LUN: %s" % lun_name)
        else:
            clean_up_info["lun_name"] = None

    if "lock_obj" in clean_up_info:
        clean_up_info["lock_obj"].release_lock(LOCK_TYPE)
        del clean_up_info["lock_obj"]
    _print("INFO: Cleaned")
    return True


def start_test(mpath_name, lun_size):
    global LOCK_TYPE, clean_up_info, lock_obj

    _print("INFO: Using %s as based multipath device" % mpath_name)
    _print("INFO: Will create new LUN with %s for this test" % lun_size)

    mp_info = mp.multipath_query_all(mpath_name)
    if not mp_info:
        _print("FAIL: Got NULL from query_mp_info(%s)" % mpath_name)
        return False

    obj_sanmgmt = sanmgmt.create_sanmgmt_for_mpath(mpath_name)
    if not obj_sanmgmt:
        _print("FAIL: Got NULL from create_sanmgmt_for_mpath(%s)" % mpath_name)
        return False

    clean_up_info["obj_sanmgmt"] = obj_sanmgmt

    scsi_disks = list(mp_info["disk"].keys())

    scsi_host_ids = []
    for scsi_disk in scsi_disks:
        host_id = scsi.scsi_host_of_scsi_name(scsi_disk)
        if host_id and host_id not in scsi_host_ids:
            scsi_host_ids.append(host_id)

    clean_up_info["lun_name"] = None
    clean_up_info["wwid"] = None
    clean_up_info["mpath_name"] = None

    lock_obj = nfs_lock.setup_nfs_lock_for_mpath(mpath_name)
    if lock_obj:
        clean_up_info["lock_obj"] = lock_obj
        _print("INFO: Requesting NFS lock (%s)..." % LOCK_TYPE)
        if not lock_obj.request_lock(LOCK_TYPE):
            _print("FAIL: Could not request NFS lock")
            return False

        _print("INFO: Waiting for NFS lock...")
        if not lock_obj.get_lock():
            _print("FAIL: Give up waiting for lock")
            sys.exit(1)
        _print("INFO: Good, got NFS lock. Start testing...")

    lun_base_name = ("io-remove-lun-%s" % time.get_time())
    lun_name = obj_sanmgmt.lun_create_and_map(size=lun_size,
                                              lun_name=lun_base_name, rescan=True)
    if not lun_name:
        _print("FAIL: Could not create LUN for test")
        _clean_up()
        return False
    clean_up_info["lun_name"] = lun_name

    _print("INFO: LUN successfully created")

    if "lun_name" not in clean_up_info:
        _print("FAIL: No LUNs created for test")
        _clean_up()
        return False

    scsi_ids = obj_sanmgmt.scsi_id_of_lun(lun_name=lun_name)
    if not scsi_ids:
        _print("FAIL: Failed to find out the SCSI ID for newly mapped LUN %s" % lun_name)
        run("ls /dev/sd*")
        print(scsi.query_all_scsi_disks())
        _clean_up()
        return False

    _print("INFO: New SCSI disks created:")
    for _ids in scsi_ids:
        _print("    %s - %s" % (_ids, scsi.get_scsi_disk_name(_ids)))

    # All disks should have the same wwid
    scsi_disk = scsi.get_scsi_disk_name(scsi_ids[0])
    wwid = scsi.wwid_of_disk(scsi_disk)
    if not wwid:
        _print("FAIL: Could not find WWID for %s" % scsi_disk)
        _clean_up()
        return False

    clean_up_info["wwid"] = wwid

    _print("INFO: New disks have WWID: %s" % wwid)

    mp.multipath_reload()

    new_mpath = mp.mpath_name_of_wwid(wwid)
    if not new_mpath:
        _print("FAIL: Could not find mpath for WWID %s" % wwid)
        mp.multipath_show()
        _clean_up()
        return False
    clean_up_info["mpath_name"] = new_mpath

    # multipath will try to queue if not path, that might cause it to hang forever
    # so force it to fail if no path
    # as we do not need sector, set it to 0 as man page says

    #    msg = "0 \"fail_if_no_path\""
    #    if not dm.dm_message_dev(new_mpath, msg):
    #        _print("FAIL: Could not disable queue_if_no_path feature")
    #        return False

    _print("INFO: Multipath enabled on tmp LUN: %s" % new_mpath)
    mp.multipath_show()

    new_mp_info = mp.multipath_query_all(new_mpath)
    if not new_mp_info:
        _print("FAIL: Could not query mp info for %s" % new_mpath)
        _clean_up()
        return False

    # Check if the multipath device is configure to queue IO device forever
    # like NetApp
    queue_io = False
    if "queue_if_no_path" in new_mp_info["feature"]:
        queue_io = True

    _print("INFO: LUN successfully found by host")

    pid = dt.dt_stress_background(of="/dev/mapper/%s" % new_mpath,
                                  time="30m", verbose=0)
    if not pid:
        _print("FAIL: Could not start DT I/O stress on %s" % new_mpath)
        _clean_up()
        return False

    linux.sleep(10)
    obj_sanmgmt.lun_remove(lun_name)

    linux.sleep(60)
    mp.multipath_show()

    # need to force IO to fail
    if queue_io:
        linux.kill_all("dt")

    _print("INFO: Waiting for dt to finish...")
    run("ps -ef | grep dt")
    # this case we will block. os.WNOHANG
    flag_dt_done, exit_status = os.waitpid(pid, 0)

    if exit_status == 0:
        _print("FAIL: I/O stress tests passed, it should have failed")
        _clean_up()
        return False

    # Wait to make sure no call trace will happen
    _print("INFO: Wait to make sure there will be no call trace")
    linux.sleep(120)
    _clean_up()
    if not log.check_all():
        _print("FAIL: detected error on LogChecker")
        return False

    _print("PASS: LUN removal with IO test")
    return True


def main():
    """
    """
    PASS = 0
    FAIL = 1

    parser = argparse.ArgumentParser(
        description='add new num')
    parser.add_argument("--mpath-name", "-m", required=False, dest="mpath_name",
                        help="Name of multipath device to use (We use it to create new device based on it)",
                        metavar="mpath")
    parser.add_argument("--lun-size", required=False, dest="lun_size", default="1GiB",
                        help="What size should be the new LUN", metavar="size. Eg: 2GiB")

    args = parser.parse_args()

    mpath_name_list = []
    if args.mpath_name:
        mpath_name_list.append(args.mpath_name)
    lun_size = args.lun_size

    # If not mpath was specified search for them on the system
    if not mpath_name_list:
        choosen_devs = sanmgmt.choose_mpaths()
        if not choosen_devs:
            _print("FAIL: Could not find any multipath device to use as base LUN")
            sys.exit(FAIL)
        mpath_name_list = list(choosen_devs.keys())

    test_pass = False
    try:
        for mpath_name in mpath_name_list:
            test_pass = start_test(mpath_name, lun_size)
    except Exception as e:
        print(e)
        traceback.print_exc()
        e = sys.exc_info()[0]
        _print("FAIL: Exception when running test (%s)" % e)
        _clean_up()
        sys.exit(FAIL)

    if test_pass:
        sys.exit(PASS)

    _clean_up()
    sys.exit(FAIL)


main()



from __future__ import absolute_import, division, print_function, unicode_literals

# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

"""beaker.py: Module to manage beaker."""

__author__ = "Bruno Goncalves"
__copyright__ = "Copyright (c) 2016 Red Hat, Inc. All rights reserved."

import re  # regex
import os
import libsan.host.linux
from libsan.host.cmdline import run


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)
    print(string)
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def update_killtime(kill_time):
    """
    Change beaker watchdog kill time
    Parameter:
    \tkill_time     new kill time in hours
    Return:
    \tTrue
    or
    \tFalse
    """
    if not is_bkr_job():
        return False

    if not kill_time:
        kill_time = 1

    result_server = os.environ['RESULT_SERVER']
    jobid = os.environ['JOBID']
    test = os.environ['TEST']
    testid = os.environ['TESTID']

    host = libsan.host.linux.hostname()
    cmd = "rhts-test-checkin %s %s %s %s %s %s" % (result_server, host, jobid, test, kill_time, testid)
    ret, output = run(cmd, return_output=True, verbose=False)
    if ret != 0:
        _print("FAIL: Could not update beaker kill time")
        print(output)
        return False
    _print("INFO: beaker_update_killtime() - Watchdog timer successfully updated to %d hours" % kill_time)
    return True


def get_task_timeout(task_id):
    """
    Get how much time the task still has
    Parameter:
    \ttask_id:          Beaker Task ID
    Return:
    \tNone:             In case of some problem
    or
    \tint(value):       The remaining time in seconds
    """
    if not is_bkr_job():
        return None

    if task_id is None:
        _print("FAIL: beaker get_task_timeout() - requires task_id as parameter")
        return None

    cmd = "bkr watchdog-show %s" % task_id
    ret, output = run(cmd, return_output=True, verbose=False)
    if ret != 0:
        _print("FAIL: beaker get_task_timeout() - Could not get beaker kill time")
        print(output)
        return None

    output_regex = re.compile(r"%s: (\d+)" % task_id)
    m = output_regex.match(output)
    if m:
        return int(m.group(1))
    _print("FAIL: beaker get_task_timeout() - Could not parse output:")
    print(output)
    return None


def log_submit(log_file):
    """
    """
    if not is_bkr_job():
        return True

    if not log_file:
        _print("FAIL: log_submit() - requires log_file parameter")
        return False

    if not os.path.exists(log_file):
        _print("FAIL: log_submit() - file does not exist" % log_file)
        return False

    cmd = "rhts-submit-log -l \"%s\"" % log_file
    ret, output = run(cmd, return_output=True, verbose=False)
    if ret != 0:
        _print("FAIL: Could not upload log %s" % log_file)
        print(output)
        return False
    _print("INFO: log_submit() - %s uploaded successfully" % log_file)
    return True


def get_task_id():
    """
    Get current task id
    Parameter:
    \tNone
    Return:
    \ttask_id:          Beaker task ID
    or
    \tNone:             Some error occurred
    """
    if not is_bkr_job():
        return None
    return os.environ['TASKID']


def get_task_status(task_id):
    """
    Requires beaker-client package installed and configured
    """
    if not is_bkr_job():
        return None

    if not task_id:
        _print("FAIL: get_task_status() - requires task ID")
        return None

    cmd = "bkr job-results --prettyxml T:%s" % task_id
    ret, output = run(cmd, return_output=True, verbose=False)
    if ret != 0:
        _print("FAIL: get_task_status() - Could not get beaker task result for T:%s" % task_id)
        print(output)
        return None

    lines = output.split("\n")
    status_regex = re.compile(r"<task.*status=\"(\S+)\"")
    for line in lines:
        m = status_regex.match(line)
        if m:
            return m.group(1)
    return None


def is_bkr_job():
    """
    Checks if is beaker job
    """
    need_env = ["RESULT_SERVER", "JOBID", "TEST", "TESTID", "TASKID"]
    for var in need_env:
        if var not in os.environ:
            return False

    result_server = os.environ['RESULT_SERVER']
    jobid = os.environ['JOBID']
    test = os.environ['TEST']
    testid = os.environ['TESTID']

    if not result_server or not jobid or not test or not testid:
        return False

    return True



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import os
import re
import fileinput
import traceback
from libsan.host.cmdline import run
import stqe.host.tc
import libsan.host.lvm as lvm
import libsan.host.loopdev as loopdev
import libsan.host.linux as linux
from six.moves import range

TestObj = None

loop_dev = {}
vg_name = "testvg"

lvm_conf_file = "/etc/lvm/lvm.conf"
bk_file = lvm_conf_file + ".copy"


def _get_conf_value(key):
    search_regex = re.compile("\W*%s\s*=\s*(\S+)" % key)
    ret = None
    for line in fileinput.input(lvm_conf_file):
        m = search_regex.match(line)
        if m:
            ret = m.group(1)
    return ret


def _meta_req_sep_pvs():
    lvm.update_config("thin_pool_metadata_require_separate_pvs", "1")
    TestObj.tok("lvcreate -l1 -T %s/pool" % vg_name)
    lv_info = lvm.lv_info("[pool_tdata]", vg_name, "devices")
    if not lv_info:
        TestObj.tfail("Could not device of tdata")
        return
    tdata_dev = lv_info["devices"]
    lv_info = lvm.lv_info("[pool_tmeta]", vg_name, "devices")
    if not lv_info:
        TestObj.tfail("Could not device of tmeta")
        return
    tmeta_dev = lv_info["devices"]
    if tdata_dev == tmeta_dev:
        TestObj.tfail("Expected tmeta an tdata to be in different device")
        run("lvs -a -o+devices")
    else:
        TestObj.tpass("tmeta an tdata are in different devices")
    # Should fail as with 3 stripes there will be no free device for separate metadata
    TestObj.tnok("lvcreate -i3 -l1 -T %s/pool2" % vg_name)

    lvm.update_config("thin_pool_metadata_require_separate_pvs", "0")
    TestObj.tok("lvcreate -l1 -T %s/pool3" % vg_name)
    TestObj.tok("lvcreate -i2 -l1 -T %s/pool4" % vg_name)
    return


def _check_default_values():
    conf = {"thin_pool_autoextend_threshold": "100",
            "thin_pool_autoextend_percent": "20",
            "thin_pool_metadata_require_separate_pvs": "0"}

    for key in conf.keys():
        if _get_conf_value(key) == conf[key]:
            TestObj.tpass("%s == '%s'" % (key, conf[key]))
        else:
            TestObj.tfail(
                "%s == '%s' and not '%s'" % (key, _get_conf_value("thin_pool_autoextend_threshold"), conf[key]))


def start_test():
    global TestObj
    global loop_dev

    print(80 * "#")
    print("INFO: Starting Thin Provisioning lvconf test")
    print(80 * "#")

    # Create 2 devices
    for dev_num in range(1, 3):
        new_dev = loopdev.create_loopdev(size=128)
        if not new_dev:
            TestObj.tfail("Could not create loop device to be used as dev%s" % dev_num)
            return False
        loop_dev["dev%s" % dev_num] = new_dev

    pvs = ""
    for dev in loop_dev.keys():
        pvs += " %s" % (loop_dev[dev])
    if not lvm.vg_create(vg_name, pvs, force=True):
        TestObj.tfail("Could not create VG")
        return False

    _meta_req_sep_pvs()
    _check_default_values()

    return True


def _clean_up():
    global loop_dev, bk_file

    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        for dev in loop_dev.keys():
            if not lvm.pv_remove(loop_dev[dev]):
                TestObj.tfail("Could not delete PV \"%s\"" % loop_dev[dev])
            linux.sleep(1)
            if not loopdev.delete_loopdev(loop_dev[dev]):
                TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])

    # restore original lvm conf file
    if os.path.isfile(bk_file):
        run("mv -f %s %s" % (bk_file, lvm_conf_file))


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    _clean_up()

    # create backup file
    run("cp -f %s %s" % (lvm_conf_file, bk_file))

    try:
        start_test()
    except Exception as e:
        print(e)
        TestObj.tfail("FAIL: Exception when running test (%s)" % traceback.format_exc())

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from time import sleep
from libsan.host.cmdline import run
from libsan.host.linux import hostname
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_var
from stqe.host.nfs_lock import release_nfs_lock_iscsi


def cleanup_iscsi():
    print("INFO: Cleaning up iSCSI.")
    errors = []

    print("Waiting for data stream to settle before logging out of iscsi.")
    sleep(5)

    disk = read_var("VDO_DEVICE").split("/").pop()
    if "brq" in hostname():
        atomic_run("Releasing NFS lock on iscsi disk %s." % disk,
                   command=release_nfs_lock_iscsi,
                   disk_name=disk,
                   errors=errors)

    commands = [
        {'message': "Logging out of iSCSI target.", 'cmd': "iscsiadm -m node -u"},
        {'message': "Cleaning targetcli config", 'cmd': "targetcli clearconfig true"},
    ]

    for cmd in commands:
        atomic_run(errors=errors,
                   command=run,
                   **cmd)

    if "brq" not in hostname():
        atomic_run("Removing image file",
                   command=run,
                   cmd="rm -rf /var/tmp/vdo-general-test.img",
                   errors=errors
                   )

    atomic_run("Cleaning var VDO_DEVICE",
               command=clean_var,
               var="VDO_DEVICE",
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = cleanup_iscsi()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var
from libsan.host.lio import TargetCLI

sub_table = {"0": "1", "1": "0"}
unsupported = ["alua_support", "block_size", "unmap_zeroes_data", "emulate_tpws", "emulate_rest_reord",
               "pi_prot_format", "emulate_tpu", "emulate_fua_read", "emulate_dpo", "pgr_support"]


def compare_attributes(dict1, dict2):
    errors = []

    for attribute in dict1:
        if int(dict1[attribute]) != int(dict2[attribute]):
            print("FAIL: " + attribute + " " + str(dict1[attribute]) +
                  " != " + dict2[attribute])
            errors.append("FAIL: " + attribute + " " + str(dict1[attribute]) +
                          " != " + dict2[attribute])

    return errors


def ramdisk_attributes_test():
    default_attr = dict()
    changed_attr = dict()

    errors = []

    name = read_var("RAMDISK_NAME")
    target = TargetCLI(path="/backstores/ramdisk/%s" % name)

    ret, data = atomic_run("Getting parameters from group attribute",
                           group="attribute",
                           command=target.get,
                           return_output=True,
                           errors=errors
                           )
    if ret != 0:
        print("FAIL: Can not get parameters for %s" % name)
        errors.append("FAIL: Can not get parameters for %s" % name)
        return errors

    attributes = data.splitlines()
    end = len(attributes)

    for attribute in attributes:
        if attributes.index(attribute) < end - 1 and "[ro]" not in attribute and \
                attributes[attributes.index(attribute) + 1].startswith("---"):
            attr = attribute.split("=")
            if attr[0] in unsupported:
                continue
            default_attr[attr[0]] = attr[1]

    for attribute in default_attr.keys():
        if int(default_attr[attribute]) == 0 or int(default_attr[attribute]) == 1:
            default_attr[attribute] = sub_table[default_attr[attribute]]
        else:
            default_attr[attribute] = int(int(default_attr[attribute]) / 2)

    atomic_run("Changing parameters in group attribute",
               group="attribute",
               command=target.set,
               errors=errors,
               **default_attr
               )

    ret, data = atomic_run("Getting parameters from group attribute",
                           group="attribute",
                           command=target.get,
                           return_output=True,
                           errors=errors,
                           **default_attr
                           )

    if ret != 0:
        print("FAIL: Can not get parameters for %s" % name)
        errors.append("FAIL: Can not get parameters for %s" % name)
        return errors

    attr_list = data.splitlines()

    for i in attr_list:
        attr = i.split("=")
        changed_attr[attr[0]] = attr[1]

    errors += compare_attributes(default_attr, changed_attr)

    return errors


if __name__ == "__main__":
    errs = ramdisk_attributes_test()
    exit(parse_ret(errs))



from __future__ import absolute_import, division, print_function, unicode_literals
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.

"""lvm.py: Module with test specific method for LVM."""

__author__ = "Bruno Goncalves"
__copyright__ = "Copyright (c) 2016 Red Hat, Inc. All rights reserved."

import re  # regex
import libsan.host.lvm as lvm
from libsan.host.cmdline import run
from stqe.host.persistent_vars import read_var
from stqe.host.fmf_tools import get_env_args


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)
    print(string)
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def check_lv_expected_value(TestObj, lv_name, vg_name, opt_val_dict):
    if not TestObj or not opt_val_dict or not lv_name or not vg_name:
        return

    opt_str = ",".join(opt_val_dict)
    lv_info = lvm.lv_info(lv_name, vg_name, options=opt_str)
    if not lv_info:
        TestObj.tfail("%s/%s does not exist" % (vg_name, lv_name))
        run("lvs -a -o +%s" % opt_str)
        return
    for opt in opt_val_dict.keys():
        if lv_info[opt] == opt_val_dict[opt]:
            TestObj.tpass("%s/%s %s == %s" % (vg_name, lv_name, opt, opt_val_dict[opt]))
            continue
        else:
            TestObj.tfail("%s/%s %s == %s, but expected %s" % (vg_name, lv_name,
                                                               opt, lv_info[opt], opt_val_dict[opt]))
            print(lv_info)
            run("lvs -a -o +%s" % opt_str)
            continue


def get_dmpd_args(dmpd_object):
    """
    Returns dict of arguments passed from FMF env vars
    :return: dict
    """
    arguments = dict(dmpd_object.arguments)
    return get_env_args(list(arguments),
                        read_var("FILE_NAMES"),
                        {})
